# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (c) 2010, 2011, 2012 Michael Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.utils as utils
import volatility.obj as obj
import volatility.plugins.common as common
import volatility.win32.tasks as tasks
import volatility.debug as debug
import volatility.plugins.registry.registryapi as registryapi
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

#--------------------------------------------------------------------------------
# vtypes
#--------------------------------------------------------------------------------

SERVICE_TYPE_FLAGS = {
            'SERVICE_KERNEL_DRIVER': 0,
            'SERVICE_FILE_SYSTEM_DRIVER': 1,
            'SERVICE_WIN32_OWN_PROCESS': 4,
            'SERVICE_WIN32_SHARE_PROCESS': 5,
            'SERVICE_INTERACTIVE_PROCESS': 8}
SERVICE_STATE_ENUM = {
            1: 'SERVICE_STOPPED',
            2: 'SERVICE_START_PENDING',
            3: 'SERVICE_STOP_PENDING',
            4: 'SERVICE_RUNNING',
            5: 'SERVICE_CONTINUE_PENDING',
            6: 'SERVICE_PAUSE_PENDING',
            7: 'SERVICE_PAUSED'}
SERVICE_START_ENUM = {
            0: 'SERVICE_BOOT_START',
            1: 'SERVICE_SYSTEM_START',
            2: 'SERVICE_AUTO_START',
            3: 'SERVICE_DEMAND_START',
            4: 'SERVICE_DISABLED'}

svcscan_base_x86 = {
    '_SERVICE_HEADER': [ None, {
    'Tag': [ 0x0, ['array', 4, ['unsigned char']]],
    'ServiceRecord': [ 0xC, ['pointer', ['_SERVICE_RECORD']]],
    } ],
    '_SERVICE_LIST_ENTRY' : [ 0x8, {
    'Blink' : [ 0x0, ['pointer', ['_SERVICE_RECORD']]],
    'Flink' : [ 0x4, ['pointer', ['_SERVICE_RECORD']]],
    } ],
    '_SERVICE_RECORD' : [ None, {
    'ServiceList' : [ 0x0, ['_SERVICE_LIST_ENTRY']],
    'ServiceName' : [ 0x8, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
    'DisplayName' : [ 0xc, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
    'Order' : [ 0x10, ['unsigned int']],
    'Tag' : [ 0x18, ['array', 4, ['unsigned char']]],
    'DriverName' : [ 0x24, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
    'ServiceProcess' : [ 0x24, ['pointer', ['_SERVICE_PROCESS']]],
    'Type' : [ 0x28, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
    'State' : [ 0x2c, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
    'Start' : [ 0x44, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
    } ],
    '_SERVICE_PROCESS' : [ None, {
    'BinaryPath' : [ 0x8, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
    'ProcessId' : [ 0xc, ['unsigned int']],
    } ],
}

svcscan_base_x64 = {
    '_SERVICE_HEADER': [ None, {
    'Tag': [ 0x0, ['array', 4, ['unsigned char']]],
    'ServiceRecord': [ 0x10, ['pointer', ['_SERVICE_RECORD']]],
    } ],
    '_SERVICE_LIST_ENTRY' : [ 0x8, {
    'Blink' : [ 0x0, ['pointer', ['_SERVICE_RECORD']]],
    'Flink' : [ 0x10, ['pointer', ['_SERVICE_RECORD']]],
    } ],
    '_SERVICE_RECORD' : [ None, {
    'ServiceList' : [ 0x0, ['_SERVICE_LIST_ENTRY']],
    'ServiceName' : [ 0x8, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
    'DisplayName' : [ 0x10, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
    'Order' : [ 0x18, ['unsigned int']],
    'Tag' : [ 0x20, ['array', 4, ['unsigned char']]],
    'DriverName' : [ 0x30, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
    'ServiceProcess' : [ 0x30, ['pointer', ['_SERVICE_PROCESS']]],
    'Type' : [ 0x38, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
    'State' : [ 0x3C, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
    'Start' : [ 0x54, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
    } ],
    '_SERVICE_PROCESS': [ None, {
    'BinaryPath': [ 0x10, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
    'ProcessId': [ 0x18, ['unsigned int']],
    } ],
}

#--------------------------------------------------------------------------------
# object Classes 
#--------------------------------------------------------------------------------

class _SERVICE_RECORD_LEGACY(obj.CType):
    "Service records for XP/2003 x86 and x64"

    @property
    def Binary(self):
        "Return the binary path for a service"

        # No path in memory for services that aren't running
        # (if needed, query the registry key)
        if str(self.State) != 'SERVICE_RUNNING':
            return obj.NoneObject("No path, service isn't running")

        # Depending on whether the service is for a process 
        # or kernel driver, the binary path is stored differently
        if 'PROCESS' in str(self.Type):
            return self.ServiceProcess.BinaryPath.dereference()
        else:
            return self.DriverName.dereference()

    @property
    def Pid(self):
        "Return the process ID for a service"

        if str(self.State) == 'SERVICE_RUNNING':
            if 'PROCESS' in str(self.Type):
                return self.ServiceProcess.ProcessId

        return obj.NoneObject("Cannot get process ID")

    def is_valid(self):
        "Check some fields for validity"
        return obj.CType.is_valid(self) and self.Order > 0 and self.Order < 0xFFFF

    def traverse(self):

        rec = self # Include this object in the list 
        while rec and rec.is_valid():
            yield rec
            rec = rec.ServiceList.Blink.dereference()

class _SERVICE_RECORD_RECENT(_SERVICE_RECORD_LEGACY):
    "Service records for 2008, Vista, 7 x86 and x64"

    def traverse(self):
        """Generator that walks the singly-linked list"""

        if self.is_valid():
            yield self # Include this object in the list 

        # Make sure we dereference these pointers, or the 
        # is_valid() checks will apply to the pointer and 
        # not the _SERVICE_RECORD object as intended. 
        rec = self.PrevEntry.dereference()
        while rec and rec.is_valid():
            yield rec
            rec = rec.PrevEntry.dereference()

class _SERVICE_HEADER(obj.CType):
    "Service headers for 2008, Vista, 7 x86 and x64"

    def is_valid(self):
        "Check some fields for validity"
        return (obj.CType.is_valid(self) and
                    self.ServiceRecord.is_valid() and
                    self.ServiceRecord.Order < 0xFFFF)

#--------------------------------------------------------------------------------
# profile modifications 
#--------------------------------------------------------------------------------

class ServiceBase(obj.ProfileModification):
    """The base applies to XP and 2003 SP0-SP1"""

    before = ['WindowsOverlay', 'WindowsObjectClasses']
    conditions = {'os': lambda x: x == 'windows'}

    def modification(self, profile):
        profile.object_classes.update({
            '_SERVICE_RECORD': _SERVICE_RECORD_LEGACY,
            '_SERVICE_HEADER': _SERVICE_HEADER,
            })
        profile.merge_overlay({'VOLATILITY_MAGIC': [ None, {
            'ServiceTag': [ 0x0, ['VolatilityMagic', dict(value = "sErv")]]
            }]})
        profile.vtypes.update(svcscan_base_x86)

class ServiceBasex64(obj.ProfileModification):
    """This overrides the base x86 vtypes with x64 vtypes"""

    before = ['WindowsOverlay', 'WindowsObjectClasses', 'ServiceBase']
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '64bit'}

    def modification(self, profile):
        profile.vtypes.update(svcscan_base_x64)

class ServiceVista(obj.ProfileModification):
    """Override the base with OC's for Vista, 2008, and 7"""

    before = ['WindowsOverlay', 'WindowsObjectClasses', 'ServiceBase']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x >= 6}

    def modification(self, profile):
        profile.object_classes.update({
            '_SERVICE_RECORD': _SERVICE_RECORD_RECENT,
            })
        profile.merge_overlay({'VOLATILITY_MAGIC': [ None, {
            'ServiceTag': [ 0x0, ['VolatilityMagic', dict(value = "serH")]]
            }]})

class ServiceVistax86(obj.ProfileModification):
    """Override the base with vtypes for x86 Vista, 2008, and 7"""

    before = ['WindowsOverlay', 'WindowsObjectClasses', 'ServiceBase']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x < 2,
                  'memory_model': lambda x: x == '32bit'}

    def modification(self, profile):
        profile.merge_overlay({'_SERVICE_RECORD': [ None, {
            'PrevEntry': [ 0x0, ['pointer', ['_SERVICE_RECORD']]],
            'ServiceName': [ 0x4, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
            'DisplayName': [ 0x8, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
            'Order': [ 0xC, ['unsigned int']],
            'ServiceProcess': [ 0x1C, ['pointer', ['_SERVICE_PROCESS']]],
            'DriverName': [ 0x1C, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
            'Type' : [ 0x20, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
            'State': [ 0x24, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
            'Start' : [ 0x3C, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
            'ServiceList' : [ 0x5C, ['_SERVICE_LIST_ENTRY']],
            }]})

class ServiceVistax64(obj.ProfileModification):
    """Override the base with vtypes for x64 Vista, 2008, and 7"""

    before = ['WindowsOverlay', 'WindowsObjectClasses', 'ServiceBase']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x < 2,
                  'memory_model': lambda x: x == '64bit'}

    def modification(self, profile):
        profile.merge_overlay({'_SERVICE_RECORD': [ None, {
            'PrevEntry': [ 0x0, ['pointer', ['_SERVICE_RECORD']]],
            'ServiceName': [ 0x8, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
            'DisplayName': [ 0x10, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
            'Order': [ 0x18, ['unsigned int']],
            'ServiceProcess': [ 0x28, ['pointer', ['_SERVICE_PROCESS']]],
            'DriverName': [ 0x28, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
            'Type' : [ 0x30, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
            'State': [ 0x34, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
            'Start' : [ 0x4C, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
            'ServiceList' : [ 0x78, ['_SERVICE_LIST_ENTRY']],
            }]})

class Service8x64(obj.ProfileModification):
    """Service structures for Win8/8.1 and Server2012/R2 64-bit"""

    before = ['WindowsOverlay', 'WindowsObjectClasses', 'ServiceBase', 'ServiceVista']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x >= 2,
                  'memory_model': lambda x: x == '64bit'}

    def modification(self, profile):
        profile.merge_overlay({
            '_SERVICE_RECORD' : [ None, {
            'Tag' : [ 0x0, ['String', dict(length = 4)]], 
            'PrevEntry': [ 0x8, ['pointer', ['_SERVICE_RECORD']]],
            'ServiceName' : [ 0x10, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
            'DisplayName' : [ 0x18, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
            'Order' : [ 0x20, ['unsigned int']],
            'DriverName' : [ 0x38, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
            'ServiceProcess' : [ 0x38, ['pointer', ['_SERVICE_PROCESS']]],
            'Type' : [ 0x40, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
            'State' : [ 0x44, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
            'Start' : [ 0x5C, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
            } ],
            '_SERVICE_PROCESS': [ None, {
            'BinaryPath': [ 0x18, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
            'ProcessId': [ 0x20, ['unsigned int']],
            } ],
        })

class Service8x86(obj.ProfileModification):
    """Service structures for Win8/8.1 32-bit"""

    before = ['WindowsOverlay', 'WindowsObjectClasses', 'ServiceBase', 'ServiceVista']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x >= 2,
                  'memory_model': lambda x: x == '32bit'}

    def modification(self, profile):
        profile.vtypes.update({
            '_SERVICE_RECORD' : [ None, {
            'Tag' : [ 0x0, ['String', dict(length = 4)]], 
            'PrevEntry': [ 0x4, ['pointer', ['_SERVICE_RECORD']]],
            'ServiceName' : [ 0x8, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
            'DisplayName' : [ 0xc, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
            'Order' : [ 0x10, ['unsigned int']],
            'DriverName' : [ 0x24, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
            'ServiceProcess' : [ 0x24, ['pointer', ['_SERVICE_PROCESS']]],
            'Type' : [ 0x28, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
            'State' : [ 0x2c, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
            'Start' : [ 0x44, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
            } ],
            '_SERVICE_PROCESS': [ None, {
            'BinaryPath': [ 0xc, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
            'ProcessId': [ 0x10, ['unsigned int']],
            } ],
        })

#--------------------------------------------------------------------------------
# svcscan plugin 
#--------------------------------------------------------------------------------

class SvcScan(common.AbstractWindowsCommand):
    "Scan for Windows services"

    def calculate(self):
        addr_space = utils.load_as(self._config)

        # Get the version we're analyzing 
        version = (addr_space.profile.metadata.get('major', 0),
                   addr_space.profile.metadata.get('minor', 0))

        tag = obj.VolMagic(addr_space).ServiceTag.v()

        # On systems more recent than XP/2003, the serH marker doesn't
        # find *all* services, but the ones it does find have linked
        # lists to the others. We use this variable to track which
        # ones we've seen so as to not yield duplicates. 
        records = []

        for task in tasks.pslist(addr_space):
            # We only want the Service Control Manager process
            if str(task.ImageFileName).lower() != "services.exe":
                continue
            # Process AS must be valid 
            process_space = task.get_process_address_space()
            if process_space == None:
                continue
            # Find all instances of the record tag 
            for address in task.search_process_memory([tag], vad_filter = lambda x: x.Length < 0x40000000):
                if version <= (5, 2):
                    # Windows XP/2003
                    rec = obj.Object("_SERVICE_RECORD", offset = address -
                            addr_space.profile.get_obj_offset('_SERVICE_RECORD', 'Tag'),
                            vm = process_space
                            )
                    # Apply our sanity checks
                    if rec.is_valid():
                        yield rec
                else:
                    # Windows Vista, 2008, and 7
                    svc_hdr = obj.Object('_SERVICE_HEADER', offset = address,
                            vm = process_space)
                    # Apply our sanity checks
                    if svc_hdr.is_valid():
                        # Since we walk the s-list backwards, if we've seen 
                        # an object, then we've also seen all objects that 
                        # exist before it, thus we can break at that time. 
                        for rec in svc_hdr.ServiceRecord.traverse():
                            if rec in records:
                                break
                            records.append(rec)
                            yield rec

    def render_dot(self, outfd, data):
        """Generate a dot graph of service relationships. 

        This currently only works for XP/2003 profiles, 
        because the linked list was removed after that.
        """

        ## Collect all the service records from calculate()
        all_services = [d for d in data] 

        ## Abort if we're not using the supported profiles
        if all_services[0].obj_vm.profile.metadata.get('major', 0) != 5:
            debug.error("This profile does not support --output=dot format")

        objects = set()
        links = set()

        for svc in all_services:
            label = "{{ {0:#x} \\n {1} \\n {2} \\n F:{3:#x} B:{4:#x} }}".format(
                                            svc.obj_offset, 
                                            svc.ServiceName.dereference(), 
                                            str(svc.State), 
                                            svc.ServiceList.Flink.v(), 
                                            svc.ServiceList.Blink.v())
            objects.add('"{0:#x}" [label="{1}" shape="record"];\n'.format(
                                            svc.obj_offset, label))

            ## Check the linked list pointers 
            flink = svc.ServiceList.Flink.dereference()
            blink = svc.ServiceList.Blink.dereference()

            if flink.is_valid():
                links.add('"{0:#x}" -> "{1:#x}" [];\n'.format(
                                            svc.obj_offset, flink.obj_offset))
            if blink.is_valid():
                links.add('"{0:#x}" -> "{1:#x}" [];\n'.format(
                                            svc.obj_offset, blink.obj_offset))

        ## Now write the graph nodes 
        outfd.write("digraph svctree { \ngraph [rankdir = \"TB\"];\n")
        for item in objects:
            outfd.write(item)
        for link in links:
            outfd.write(link)
        outfd.write("}\n")

    @staticmethod
    def get_service_info(regapi):
        ccs = regapi.reg_get_currentcontrolset()
        key_name = "{0}\\services".format(ccs)
        info = {}
        for subkey in regapi.reg_get_all_subkeys(hive_name = "system", key = key_name):

            path_value = ""
            dll_value = ""

            image_path = regapi.reg_get_value(hive_name = "system", key = "", value = "ImagePath", given_root = subkey)
            if image_path:
                path_value = utils.remove_unprintable(image_path)

            for rootkey in regapi.reg_get_all_subkeys(hive_name = "system", key = "", given_root = subkey):
                if rootkey.Name == "Parameters":
                    service_dll = regapi.reg_get_value(hive_name = "system", key = "", value = "ServiceDll", given_root = rootkey)
                    if service_dll != None:
                        dll_value = utils.remove_unprintable(service_dll)
                    break

            info[utils.remove_unprintable(str(subkey.Name))] = (dll_value, path_value)

        return info

    def unified_output(self, data):
        if self._config.VERBOSE:
            return TreeGrid([("Offset", Address),
                       ("Order", int),
                       ("Start", str),
                       ("PID", int),
                       ("ServiceName", str),
                       ("DisplayName", str),
                       ("ServiceType", str),
                       ("State", str),
                       ("BinaryPath", str),
                       ("ServiceDll", str),
                       ("ImagePath", str)],
                        self.generator(data)) 

        return TreeGrid([("Offset", Address),
                       ("Order", int),
                       ("Start", str),
                       ("PID", int),
                       ("ServiceName", str),
                       ("DisplayName", str),
                       ("ServiceType", str),
                       ("State", str),
                       ("BinaryPath", str)],
                        self.generator(data))

    def generator(self, data):
        if self._config.VERBOSE:
            regapi = registryapi.RegistryApi(self._config)
            info = self.get_service_info(regapi)

        for rec in data:
            if self._config.VERBOSE:
                vals = info.get("{0}".format(rec.ServiceName.dereference()), None)
                yield (0, [Address(rec.obj_offset),
                        int(rec.Order),
                        str(rec.Start),
                        int(rec.Pid),
                        str(rec.ServiceName.dereference() or ""),
                        str(rec.DisplayName.dereference() or ""),
                        str(rec.Type),
                        str(rec.State),
                        str(rec.Binary or ""),
                        str(val[0] if vals else ""),
                        str(val[1] if vals else "")])  
            else:
                yield (0, [Address(rec.obj_offset),
                        int(rec.Order),
                        str(rec.Start),
                        int(rec.Pid),
                        str(rec.ServiceName.dereference() or ""),
                        str(rec.DisplayName.dereference() or ""),
                        str(rec.Type),
                        str(rec.State),
                        str(rec.Binary or "")])

    def render_text(self, outfd, data):

        if self._config.VERBOSE:
            regapi = registryapi.RegistryApi(self._config)
            info = self.get_service_info(regapi)

        for rec in data:
            # This can't possibly look neat in a table with columns...
            outfd.write("Offset: {0:#x}\n".format(rec.obj_offset))
            outfd.write("Order: {0}\n".format(rec.Order))
            outfd.write("Start: {0}\n".format(rec.Start))
            outfd.write("Process ID: {0}\n".format(rec.Pid))
            outfd.write("Service Name: {0}\n".format(rec.ServiceName.dereference()))
            outfd.write("Display Name: {0}\n".format(rec.DisplayName.dereference()))
            outfd.write("Service Type: {0}\n".format(rec.Type))
            outfd.write("Service State: {0}\n".format(rec.State))
            outfd.write("Binary Path: {0}\n".format(rec.Binary))

            if self._config.VERBOSE:
                vals = info.get("{0}".format(rec.ServiceName.dereference()), None)
                if vals:
                    outfd.write("ServiceDll: {0}\n".format(vals[0]))
                    outfd.write("ImagePath: {0}\n".format(vals[1]))

            outfd.write("\n")



# Volatility
# Copyright (C) 2009-2012 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#
from volatility import renderers
from volatility.commands import Command

import volatility.plugins.crashinfo as crashinfo
from volatility.renderers.basic import Address, Hex

class VBoxInfo(crashinfo.CrashInfo):
    """Dump virtualbox information"""

    target_as = ['VirtualBoxCoreDumpElf64']

    def unified_output(self, data):
        return renderers.TreeGrid([("FileOffset", Address),
                                 ("Memory Offset", Address),
                                 ("Size", Hex)],
                                  self.generator(data))

    def generator(self, data):
        for memory_offset, file_offset, length in data.get_runs():
            yield (0, [Address(file_offset),
                                  Address(memory_offset),
                                  Hex(length)])

    def render_text(self, outfd, data):

        header = data.get_header()

        outfd.write("Magic: {0:#x}\n".format(header.u32Magic))
        outfd.write("Format: {0:#x}\n".format(header.u32FmtVersion))
        outfd.write("VirtualBox {0}.{1}.{2} (revision {3})\n".format(
                header.Major,
                header.Minor, header.Build,
                header.u32VBoxRevision))
        outfd.write("CPUs: {0}\n\n".format(header.cCpus))

        Command.render_text(self, outfd, data)
    
class QemuInfo(VBoxInfo):
    """Dump Qemu information"""

    target_as = ['QemuCoreDumpElf']

    def render_text(self, outfd, data):
        Command.render_text(self, outfd, data)


# Volatility
# Copyright (c) 2015 Michael Ligh (michael.ligh@mnin.org)
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import sys 
import volatility.plugins.taskmods as taskmods 
import libapi 

def main():

    ## sys.argv[1] = volatility profile 
    ## sys.argv[2] = full path on disk to your memory sample

    config = libapi.get_config(sys.argv[1], sys.argv[2])
    data = libapi.get_json(config, taskmods.PSList)

    ## `data` now contains json with two keys: `columns` and `rows`, where `columns`
    ## contains a list of column headings (matching the corresponding volatility 
    ## plugin output) and `rows` contains a list of the values for each object found.

    ## you can either print/save all columns, or you can drill down to a particular 
    ## column by getting the desired column's index as shown below and then accessing
    ## the index in each row. the following example prints each process' name. 
    
    name_index = data['columns'].index('Name')

    for row in data['rows']:
        print row[name_index]

if __name__ == "__main__":
    main()


# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.utils as utils
import volatility.plugins.linux.common as linux_common
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class linux_pslist(linux_common.AbstractLinuxCommand):
    """Gather active tasks by walking the task_struct->task list"""

    def __init__(self, config, *args, **kwargs):
        linux_common.AbstractLinuxCommand.__init__(self, config, *args, **kwargs)
        config.add_option('PID', short_option = 'p', default = None,
                          help = 'Operate on these Process IDs (comma-separated)',
                          action = 'store', type = 'str')

    @staticmethod
    def virtual_process_from_physical_offset(addr_space, offset):
        pspace = utils.load_as(addr_space.get_config(), astype = 'physical')
        task = obj.Object("task_struct", vm = pspace, offset = offset)
        parent = obj.Object("task_struct", vm = addr_space, offset = task.parent)
        
        for child in parent.children.list_of_type("task_struct", "sibling"):
            if child.obj_vm.vtop(child.obj_offset) == task.obj_offset:
                return child
        
        return obj.NoneObject("Unable to bounce back from task_struct->parent->task_struct")

    def allprocs(self):
        linux_common.set_plugin_members(self)

        init_task_addr = self.addr_space.profile.get_symbol("init_task")
        init_task = obj.Object("task_struct", vm = self.addr_space, offset = init_task_addr)

        # walk the ->tasks list, note that this will *not* display "swapper"
        for task in init_task.tasks:
            yield task

    def calculate(self):
        linux_common.set_plugin_members(self)

        pidlist = self._config.PID
        if pidlist:
            pidlist = [int(p) for p in self._config.PID.split(',')]

        for task in self.allprocs():
            if not pidlist or task.pid in pidlist:
                yield task

    def unified_output(self, data):
        return TreeGrid([("Offset", Address),
                       ("Name", str),
                       ("Pid", int),
                       ("Uid", str),
                       ("Gid", str),
                       ("DTB", Address),
                       ("StartTime", str)],
                        self.generator(data))

    def generator(self, data):
        for task in data:
            if task.mm.pgd == None:
                dtb = task.mm.pgd
            else:
                dtb = self.addr_space.vtop(task.mm.pgd) or task.mm.pgd
            yield (0, [Address(task.obj_offset),
                                  str(task.comm),
                                  int(task.pid),
                                  str(task.uid) if task.uid else "-",
                                  str(task.gid) if task.gid else "-",
                                  Address(dtb),
                                  str(task.get_task_start_time())])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Offset", "[addrpad]"),
                                  ("Name", "20"),
                                  ("Pid", "15"),
                                  ("Uid", "15"),
                                  ("Gid", "6"),
                                  ("DTB", "[addrpad]"),
                                  ("Start Time", "")])
        for task in data:
            if task.mm.pgd == None:
                dtb = task.mm.pgd
            else:
                dtb = self.addr_space.vtop(task.mm.pgd) or task.mm.pgd
            self.table_row(outfd, task.obj_offset,
                                  task.comm,
                                  str(task.pid),
                                  str(task.uid) if task.uid else "-",
                                  str(task.gid) if task.gid else "-",
                                  dtb,
                                  task.get_task_start_time())


class linux_memmap(linux_pslist):
    """Dumps the memory map for linux tasks"""

    def unified_output(self, data):
        return TreeGrid([("Task", str),
                       ("Pid", int),
                       ("Virtual", Address),
                       ("Physical", Address),
                       ("Size", Address)],
                        self.generator(data))

    def generator(self, data):
        for task in data:
            task_space = task.get_process_address_space()

            pagedata = task_space.get_available_pages()
            if pagedata:
                for p in pagedata:
                    pa = task_space.vtop(p[0])
                    # pa can be 0, according to the old memmap, but can't == None(NoneObject)
                    if pa != None:
                        yield (0, [str(task.comm), int(task.pid), Address(p[0]), Address(pa), Address(p[1])])
            else:
                yield(0, [str(task.comm), int(task.pid), Address(-1), Address(-1), Address(-1)])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Task", "16"),
                                  ("Pid", "8"),
                                  ("Virtual", "[addrpad]"),
                                  ("Physical", "[addrpad]"),
                                  ("Size", "[addr]")])

        for task in data:
            task_space = task.get_process_address_space()

            pagedata = task_space.get_available_pages()
            if pagedata:
                for p in pagedata:
                    pa = task_space.vtop(p[0])
                    # pa can be 0, according to the old memmap, but can't == None(NoneObject)
                    if pa != None:
                        self.table_row(outfd, task.comm, task.pid, p[0], pa, p[1])
                    #else:
                    #    outfd.write("0x{0:10x} 0x000000     0x{1:12x}\n".format(p[0], p[1]))
            else:
                outfd.write("Unable to read pages for {0} pid {1}.\n".format(task.comm, task.pid))



# Volatility
#
# Authors:
# Michael Hale Ligh <michael.hale@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.utils as utils
import volatility.plugins.common as common
import volatility.obj as obj
import volatility.cache as cache
import volatility.debug as debug
import volatility.poolscan as poolscan
import socket
import volatility.plugins.overlays.windows.tcpip_vtypes as tcpip_vtypes
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

# Python's socket.AF_INET6 is 0x1e but Microsoft defines it 
# as a constant value of 0x17 in their source code. Thus we 
# need Microsoft's since that's what is found in memory.
AF_INET = 2
AF_INET6 = 0x17

# String representations of INADDR_ANY and INADDR6_ANY
inaddr_any = utils.inet_ntop(socket.AF_INET, '\0' * 4)
inaddr6_any = utils.inet_ntop(socket.AF_INET6, '\0' * 16)

#--------------------------------------------------------------------------------
# pool scanners 
#--------------------------------------------------------------------------------

class PoolScanUdpEndpoint(poolscan.PoolScanner):
    """PoolScanner for Udp Endpoints"""

    def __init__(self, address_space):
        poolscan.PoolScanner.__init__(self, address_space)

        self.pooltag = "UdpA"
        self.struct_name = "_UDP_ENDPOINT"

        self.checks = [('CheckPoolSize', dict(condition = lambda x: x >= 0xa8)),
                   ('CheckPoolType', dict(non_paged = True, free = True)),
                   ('CheckPoolIndex', dict(value = lambda x : x < 5)),
                   ]

class PoolScanTcpListener(poolscan.PoolScanner):
    """PoolScanner for Tcp Listeners"""

    def __init__(self, address_space):
        poolscan.PoolScanner.__init__(self, address_space)

        self.pooltag = "TcpL"
        self.struct_name = "_TCP_LISTENER"

        self.checks = [('CheckPoolSize', dict(condition = lambda x: x >= 0xa8)),
                   ('CheckPoolType', dict(non_paged = True, free = True)),
                   ('CheckPoolIndex', dict(value = lambda x : x < 5)),
                   ]

class PoolScanTcpEndpoint(poolscan.PoolScanner):
    """PoolScanner for TCP Endpoints"""

    def __init__(self, address_space):
        poolscan.PoolScanner.__init__(self, address_space)

        self.pooltag = "TcpE"
        self.struct_name = "_TCP_ENDPOINT"

        self.checks = [('CheckPoolSize', dict(condition = lambda x: x >= 0x1f0)),
                   ('CheckPoolType', dict(non_paged = True, free = True)),
                   ('CheckPoolIndex', dict(value = lambda x : x < 5)),
                   ]

#--------------------------------------------------------------------------------
# object classes 
#--------------------------------------------------------------------------------

class _TCP_LISTENER(obj.CType):
    """Class for objects found in TcpL pools"""

    @property
    def AddressFamily(self):
        return self.InetAF.dereference().AddressFamily

    @property
    def Owner(self):
        return self.m('Owner').dereference()

    def dual_stack_sockets(self):
        """Handle Windows dual-stack sockets"""

        # If this pointer is valid, the socket is bound to 
        # a specific IP address. Otherwise, the socket is 
        # listening on all IP addresses of the address family. 
        local_addr = self.LocalAddr.dereference()

        # Note the remote address is always INADDR_ANY or 
        # INADDR6_ANY for sockets. The moment a client 
        # connects to the listener, a TCP_ENDPOINT is created
        # and that structure contains the remote address.
        if local_addr != None:
            inaddr = local_addr.pData.dereference().dereference()
            if self.AddressFamily == AF_INET:
                yield "v4", inaddr.addr4, inaddr_any
            else:
                yield "v6", inaddr.addr6, inaddr6_any
        else:
            yield "v4", inaddr_any, inaddr_any
            if self.AddressFamily == AF_INET6:
                yield "v6", inaddr6_any, inaddr6_any

    def is_valid(self):
        return obj.CType.is_valid(self) and self.AddressFamily in (AF_INET, AF_INET6)

class _TCP_ENDPOINT(_TCP_LISTENER):
    """Class for objects found in TcpE pools"""

    def _ipv4_or_ipv6(self, in_addr):

        if self.AddressFamily == AF_INET:
            return in_addr.addr4
        else:
            return in_addr.addr6

    @property
    def LocalAddress(self):
        inaddr = self.AddrInfo.dereference().Local.\
                            pData.dereference().dereference()

        return self._ipv4_or_ipv6(inaddr)

    @property
    def RemoteAddress(self):
        inaddr = self.AddrInfo.dereference().\
                            Remote.dereference()

        return self._ipv4_or_ipv6(inaddr)

    def is_valid(self):
        if not obj.CType.is_valid(self):
            return False
  
        if self.AddressFamily not in (AF_INET, AF_INET6):
            return False
 
        if (self.State.v() not in tcpip_vtypes.TCP_STATE_ENUM or
                    (not self.LocalAddress and (not self.Owner or
                    self.Owner.UniqueProcessId == 0 or
                    self.Owner.UniqueProcessId > 65535))):
            return False

        return True

class _UDP_ENDPOINT(_TCP_LISTENER):
    """Class for objects found in UdpA pools"""

#--------------------------------------------------------------------------------
# profile modifications 
#--------------------------------------------------------------------------------

class NetscanObjectClasses(obj.ProfileModification):
    """Network OCs for Vista, 2008, and 7 x86 and x64"""

    before = ['WindowsObjectClasses']

    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x : x == 6,
                  'minor': lambda x : x >= 0}

    def modification(self, profile):
        profile.object_classes.update({
            '_TCP_LISTENER': _TCP_LISTENER,
            '_TCP_ENDPOINT': _TCP_ENDPOINT,
            '_UDP_ENDPOINT': _UDP_ENDPOINT,
            })

#--------------------------------------------------------------------------------
# netscan plugin 
#--------------------------------------------------------------------------------

class Netscan(common.AbstractScanCommand):
    """Scan a Vista (or later) image for connections and sockets"""

    scanners = [PoolScanUdpEndpoint, PoolScanTcpListener, PoolScanTcpEndpoint]

    @staticmethod
    def is_valid_profile(profile):
        return (profile.metadata.get('os', 'unknown') == 'windows' and
                profile.metadata.get('major', 0) == 6)

    def calculate(self):
        addr_space = utils.load_as(self._config)

        if not self.is_valid_profile(addr_space.profile):
            debug.error("This command does not support the selected profile.")

        for objct in self.scan_results(addr_space):

            if isinstance(objct, _UDP_ENDPOINT):
                # For UdpA, the state is always blank and the remote end is asterisks
                for ver, laddr, _ in objct.dual_stack_sockets():
                    yield objct, "UDP" + ver, laddr, objct.Port, "*", "*", ""
            elif isinstance(objct, _TCP_ENDPOINT):

                if objct.AddressFamily == AF_INET:
                    proto = "TCPv4"
                elif objct.AddressFamily == AF_INET6:
                    proto = "TCPv6"

                yield objct, proto, objct.LocalAddress, objct.LocalPort, \
                    objct.RemoteAddress, objct.RemotePort, objct.State
            elif isinstance(objct, _TCP_LISTENER):
                # For TcpL, the state is always listening and the remote port is zero
                for ver, laddr, raddr in objct.dual_stack_sockets():
                    yield objct, "TCP" + ver, laddr, objct.Port, raddr, 0, "LISTENING"

    def unified_output(self, data):
        return TreeGrid([(self.offset_column(), Address),
                       ("Proto", str),
                       ("LocalAddr", str),
                       ("ForeignAddr", str),
                       ("State", str),
                       ("PID", int),
                       ("Owner", str),
                       ("Created", str)],
                        self.generator(data))

    def generator(self, data):
        for net_object, proto, laddr, lport, raddr, rport, state in data:

            lendpoint = "{0}:{1}".format(laddr, lport)
            rendpoint = "{0}:{1}".format(raddr, rport)
            pid = -1
            owner = ""
            if net_object.Owner != None:
                pid = int(net_object.Owner.UniqueProcessId)
                owner = str(net_object.Owner.ImageFileName)

            yield (0, 
                [Address(net_object.obj_offset), 
                str(proto), 
                lendpoint,
                rendpoint, 
                str(state), 
                pid,
                owner,
                str(net_object.CreateTime or '')])

    def render_text(self, outfd, data):
        outfd.write("{0:<18} {1:<8} {2:<30} {3:<20} {4:<16} {5:<8} {6:<14} {7}\n".format(
            self.offset_column(), "Proto", "Local Address", "Foreign Address",
            "State", "Pid", "Owner", "Created"))

        for net_object, proto, laddr, lport, raddr, rport, state in data:
            lendpoint = "{0}:{1}".format(laddr, lport)
            rendpoint = "{0}:{1}".format(raddr, rport)
            pid = -1
            owner = ""
            if net_object.Owner != None:
                pid = int(net_object.Owner.UniqueProcessId)
                owner = str(net_object.Owner.ImageFileName)

            outfd.write("{0:<#18x} {1:<8} {2:<30} {3:<20} {4:<16} {5:<8} {6:<14} {7}\n".format(
                net_object.obj_offset, proto, lendpoint,
                rendpoint, state, pid,
                owner,
                str(net_object.CreateTime or '')
                ))



# Volatility
# Copyright (c) 2008-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       The Volatility Foundation
@license:      GNU General Public License 2.0
@contact:      awalters@4tphi.net

This file provides support for Windows 8.
"""

import struct
import volatility.plugins.overlays.windows.windows as windows
import volatility.obj as obj
import volatility.constants as constants
import volatility.utils as utils
import volatility.debug as debug #pylint: disable-msg=W0611
import volatility.addrspace as addrspace
import volatility.plugins.malware.malfind as malfind
import volatility.plugins.overlays.windows.pe_vtypes as pe_vtypes
import volatility.plugins.overlays.windows.ssdt_vtypes as ssdt_vtypes
import volatility.plugins.overlays.windows.win7 as win7
import volatility.plugins.overlays.windows.vista as vista

try:
    import distorm3
    has_distorm = True
except:
    has_distorm = False

class _HANDLE_TABLE32(windows._HANDLE_TABLE):
    """A class for 32-bit Windows 8 handle tables"""    

    @property
    def HandleCount(self):
        """The Windows 8 / 2012 handle table does not have a 
        HandleCount member, so we fake it. 

        Alternately, we could return len(self.handles()) and
        show a valid number in pslist, however pslist would 
        be much slower than normal.
        """

        return 0

    def get_item(self, entry, handle_value = 0):
        """Returns the OBJECT_HEADER of the associated handle. 
        The parent is the _HANDLE_TABLE_ENTRY so that an object
        can be linked to its GrantedAccess.
        """

        if entry.InfoTable == 0:
            return obj.NoneObject("LeafHandleValue pointer is invalid")

        return obj.Object("_OBJECT_HEADER", 
                          offset = entry.InfoTable & ~7, 
                          vm = self.obj_vm, 
                          parent = entry, 
                          handle_value = handle_value)

class _HANDLE_TABLE64(_HANDLE_TABLE32):
    """A class for 64-bit Windows 8 / 2012 handle tables"""   

    DECODE_MAGIC = 0x13

    def decode_pointer(self, value):
        """Decode a pointer like SAR. Since Python does not 
        have an operator for shift arithmetic, we implement
        one ourselves.
        """

        value = value & 0xFFFFFFFFFFFFFFF8
        value = value >> self.DECODE_MAGIC
        if (value & 1 << 44):
            return value | 0xFFFFF00000000000
        else:
            return value | 0xFFFF000000000000

    def get_item(self, entry, handle_value = 0):
        """Returns the OBJECT_HEADER of the associated handle. 
        The parent is the _HANDLE_TABLE_ENTRY so that an object
        can be linked to its GrantedAccess.
        """

        if entry.LowValue == 0:
            return obj.NoneObject("LowValue pointer is invalid")

        return obj.Object("_OBJECT_HEADER", 
                          offset = self.decode_pointer(entry.LowValue), 
                          vm = self.obj_vm, 
                          parent = entry, 
                          handle_value = handle_value)

class _HANDLE_TABLE_81R264(_HANDLE_TABLE64):
    """A class for 64-bit Windows 8.1 / 2012 R2 handle tables"""   
    DECODE_MAGIC = 0x10

class _PSP_CID_TABLE32(_HANDLE_TABLE32):
    """PspCidTable for 32-bit Windows 8"""

class _PSP_CID_TABLE64(_HANDLE_TABLE64):
    """PspCidTable for 64-bit Windows 8 and Server 2012"""

    def get_item(self, entry, handle_value = 0):
        """Starting with 8/2012 x64 the PsPCidTable pointers
        go directly to an object rather than an object header.
        """

        if entry.LowValue == 0:
            return obj.NoneObject("LowValue pointer is invalid")

        body_offset = self.obj_vm.profile.get_obj_offset("_OBJECT_HEADER", "Body")
        head_offset = self.decode_pointer(entry.LowValue) - body_offset

        return obj.Object("_OBJECT_HEADER", 
                          offset = head_offset, 
                          vm = self.obj_vm, 
                          parent = entry, 
                          handle_value = handle_value)

class _PSP_CID_TABLE_81R264(_PSP_CID_TABLE64):
    """PspCidTable for 64-bit Windows 8.1 and Server 2012 R2"""
    DECODE_MAGIC = 0x10

class _LDR_DATA_TABLE_ENTRY(pe_vtypes._LDR_DATA_TABLE_ENTRY):
    """A class for DLL modules"""
    
    @property
    def LoadCount(self):
        """The Windows 8 / 2012 module does not have a 
        LoadCount member, so we fake it.
        """

        return 0

class _OBJECT_HEADER(win7._OBJECT_HEADER):
    """A class for object headers on Win 8 / Server 2012"""

    # This specifies the order the headers are found below the _OBJECT_HEADER
    # Note the AuditInfo field which is new as of Windows 8 / 2012
    optional_header_mask = (('CreatorInfo', '_OBJECT_HEADER_CREATOR_INFO', 0x01),
                            ('NameInfo', '_OBJECT_HEADER_NAME_INFO', 0x02),
                            ('HandleInfo', '_OBJECT_HEADER_HANDLE_INFO', 0x04),
                            ('QuotaInfo', '_OBJECT_HEADER_QUOTA_INFO', 0x08),
                            ('ProcessInfo', '_OBJECT_HEADER_PROCESS_INFO', 0x10),
                            ('AuditInfo', '_OBJECT_HEADER_AUDIT_INFO', 0x40),
                            )

    type_map = { 2: 'Type',
                3: 'Directory',
                4: 'SymbolicLink',
                5: 'Token',
                6: 'Job',
                7: 'Process',
                8: 'Thread',
                9: 'UserApcReserve',
                10: 'IoCompletionReserve',
                11: 'DebugObject',
                12: 'Event',
                13: 'EventPair',
                14: 'Mutant',
                15: 'Callback',
                16: 'Semaphore',
                17: 'Timer',
                18: 'IRTimer',
                19: 'Profile',
                20: 'KeyedEvent',
                21: 'WindowStation',
                22: 'Desktop',
                24: 'TpWorkerFactory',
                25: 'Adapter',
                26: 'Controller',
                27: 'Device',
                28: 'Driver',
                29: 'IoCompletion',
                30: 'WaitCompletionPacket',
                31: 'File',
                32: 'TmTm',
                33: 'TmTx',
                34: 'TmRm',
                35: 'TmEn',
                36: 'Section',
                37: 'Session',
                38: 'Key',
                39: 'ALPC Port',
                40: 'PowerRequest',
                41: 'WmiGuid',
                42: 'EtwRegistration',
                43: 'EtwConsumer',
                44: 'FilterConnectionPort',
                45: 'FilterCommunicationPort',
                46: 'PcwObject',
                47: 'DxgkSharedResource', 
                48: 'DxgkSharedSyncObject',
            }

    @property
    def GrantedAccess(self):
        """Return the object's granted access permissions"""

        if self.obj_parent:
            return self.obj_parent.GrantedAccessBits
        return obj.NoneObject("No parent known")


    def is_valid(self):
        """Determine if a given object header is valid"""

        if not obj.CType.is_valid(self):
            return False

        if self.InfoMask > 0x48:
            return False

        if self.PointerCount > 0x1000000 or self.PointerCount < 0:
            return False

        return True

class _OBJECT_HEADER_81R2(_OBJECT_HEADER):
    """A class for object headers on Win 8.1 / Server 2012 R2"""

    type_map = { 2: 'Type',
                3: 'Directory',
                4: 'SymbolicLink',
                5: 'Token',
                6: 'Job',
                7: 'Process',
                8: 'Thread',
                9: 'UserApcReserve',
                10: 'IoCompletionReserve',
                11: 'DebugObject',
                12: 'Event',
                13: 'Mutant',
                14: 'Callback',
                15: 'Semaphore',
                16: 'Timer',
                17: 'IRTimer',
                18: 'Profile',
                19: 'KeyedEvent',
                20: 'WindowStation',
                21: 'Desktop',
                22: 'Composition',
                23: 'TpWorkerFactory',
                24: 'Adapter',
                25: 'Controller',
                26: 'Device',
                27: 'Driver',
                28: 'IoCompletion',
                29: 'WaitCompletionPacket',
                30: 'File',
                31: 'TmTm',
                32: 'TmTx',
                33: 'TmRm',
                34: 'TmEn',
                35: 'Section',
                36: 'Session',
                37: 'Key',
                38: 'ALPC Port',
                39: 'PowerRequest',
                40: 'WmiGuid',
                41: 'EtwRegistration',
                42: 'EtwConsumer',
                43: 'FilterConnectionPort',
                44: 'FilterCommunicationPort',
                45: 'PcwObject',
                46: 'DxgkSharedResource',
            }

class Win8KDBG(windows.AbstractKDBGMod):
    """The Windows 8 / 2012 KDBG signatures"""

    before = ['WindowsOverlay']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x >= 2}

    kdbgsize = 0x360

    def modification(self, profile):

        if profile.metadata.get('memory_model', '32bit') == '32bit':
            signature = '\x00\x00\x00\x00\x00\x00\x00\x00'
        else:
            signature = '\x03\xf8\xff\xff'
        signature += 'KDBG' + struct.pack('<H', self.kdbgsize)

        profile.merge_overlay({
            'VOLATILITY_MAGIC': [ None, {
            'KDBGHeader': [ None, ['VolatilityMagic', dict(value = signature)]]
            }]})

class Win8x86DTB(obj.ProfileModification):
    """The Windows 8 32-bit DTB signature"""

    before = ['WindowsOverlay']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x >= 2,
                  'memory_model': lambda x: x == '32bit',
                  }

    def modification(self, profile):
        profile.merge_overlay({
            'VOLATILITY_MAGIC': [ None, {
            'DTBSignature' : [ None, ['VolatilityMagic', dict(value = "\x03\x00\x28\x00")]],
            }]})

class Win8x64MaxCommit(obj.ProfileModification):
    """The Windows 8 / Server 2012 MM_MAX_COMMIT value"""

    before = ["Windows64Overlay"]
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x >= 2,
                  'memory_model': lambda x: x == '64bit',
                  }

    def modification(self, profile):
        profile.merge_overlay({
            'VOLATILITY_MAGIC': [ 0x0, {
            'MM_MAX_COMMIT': [ 0x0, ['VolatilityMagic', dict(value = 0x7fffffff)]],
             }]})

class Win8x64DTB(obj.ProfileModification):
    """The Windows 8 32-bit DTB signature"""

    before = ['WindowsOverlay', 'Windows64Overlay']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x >= 2,
                  'memory_model': lambda x: x == '64bit',
                  }

    def modification(self, profile):
        profile.merge_overlay({
            'VOLATILITY_MAGIC': [ None, {
            'DTBSignature' : [ None, ['VolatilityMagic', dict(value = "\x03\x00\xb2\x00")]],
            }]})

class Win8x86SyscallVTypes(obj.ProfileModification):
    """Applying the SSDT structures for Win 8 32-bit"""

    before = ['WindowsVTypes']
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '32bit',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x >= 2}

    def modification(self, profile):
        # Same as 2003, which basically just means there are
        # only two SSDT tables by default. 
        profile.vtypes.update(ssdt_vtypes.ssdt_vtypes_2003)

class Win8ObjectClasses(obj.ProfileModification):
    before = ["WindowsObjectClasses", "Win7ObjectClasses", "WinPEObjectClasses", "MalwarePspCid"]
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x >= 2}

    def modification(self, profile):

        memory_model = profile.metadata.get("memory_model", "32bit") 
        major = profile.metadata.get("major", 0)
        minor = profile.metadata.get("minor", 0)

        if memory_model == '32bit':
            handletable = _HANDLE_TABLE32
            pspcidtable = _PSP_CID_TABLE32
        else:
            if (major, minor) >= (6, 3):
                handletable = _HANDLE_TABLE_81R264
                pspcidtable = _PSP_CID_TABLE_81R264
            else:
                handletable = _HANDLE_TABLE64
                pspcidtable = _PSP_CID_TABLE64

        if (major, minor) == (6, 3):
            objheader = _OBJECT_HEADER_81R2
        else:
            objheader = _OBJECT_HEADER

        profile.object_classes.update({
                "_LDR_DATA_TABLE_ENTRY": _LDR_DATA_TABLE_ENTRY, 
                "_HANDLE_TABLE": handletable,
                "_OBJECT_HEADER": objheader,
                "_PSP_CID_TABLE": pspcidtable,
                })

class Win8SP0x64(obj.Profile):
    """ A Profile for Windows 8 x64 """
    _md_memory_model = '64bit'
    _md_os = 'windows'
    _md_major = 6
    _md_minor = 2
    _md_build = 9200
    _md_vtype_module = 'volatility.plugins.overlays.windows.win8_sp0_x64_vtypes'

class Win8SP1x64(obj.Profile):
    """ A Profile for Windows 8.1 x64 """
    _md_memory_model = '64bit'
    _md_os = 'windows'
    _md_major = 6
    _md_minor = 3
    _md_build = 9600
    _md_vtype_module = 'volatility.plugins.overlays.windows.win8_sp1_x64_vtypes'

class Win2012x64(Win8SP0x64):
    """ A Profile for Windows Server 2012 x64 """
    _md_build = 9201 ##FIXME: fake build number to indicate server 2012 vs windows 8

class Win2012R2x64(Win8SP1x64):
    """ A Profile for Windows Server 2012 R2 x64 """
    _md_build = 9601 ##FIXME: fake build number to indicate server 2012 R2 vs windows 8.1

class Win8SP0x86(obj.Profile):
    """ A Profile for Windows 8 x86 """
    _md_memory_model = '32bit'
    _md_os = 'windows'
    _md_major = 6
    _md_minor = 2
    _md_build = 9200
    _md_vtype_module = 'volatility.plugins.overlays.windows.win8_sp0_x86_vtypes'

class Win8SP1x86(obj.Profile):
    """ A Profile for Windows 8.1 x86 """
    _md_memory_model = '32bit'
    _md_os = 'windows'
    _md_major = 6
    _md_minor = 3
    _md_build = 9600
    _md_vtype_module = 'volatility.plugins.overlays.windows.win8_sp1_x86_vtypes'

class Win81U1x64(obj.Profile):
    """ A Profile for Windows 8.1 Update 1 x64 """
    _md_memory_model = '64bit'
    _md_os = 'windows'
    _md_major = 6
    _md_minor = 3
    _md_build = 17031 
    _md_vtype_module = 'volatility.plugins.overlays.windows.win81_u1_x64_vtypes'

class Win81U1x86(obj.Profile):
    """ A Profile for Windows 8.1 Update 1 x86 """
    _md_memory_model = '32bit'
    _md_os = 'windows'
    _md_major = 6
    _md_minor = 3
    _md_build = 17031 
    _md_vtype_module = 'volatility.plugins.overlays.windows.win81_u1_x86_vtypes'



import volatility.plugins.common as common
import volatility.utils as utils
import volatility.plugins.filescan as filescan
import volatility.plugins.modscan as modscan
import volatility.plugins.gui.atoms as atoms
import volatility.plugins.gui.windowstations as windowstations
import volatility.plugins.sockscan as sockscan
import volatility.plugins.connscan as connscan
import volatility.plugins.netscan as netscan
import volatility.plugins.malware.callbacks as callbacks

class MultiScan(common.AbstractScanCommand):
    """Scan for various objects at once"""

    def __init__(self, config, *args, **kwargs):
        common.AbstractScanCommand.__init__(self, config, *args, **kwargs)

        self.scanners = [
            filescan.PoolScanFile,
            filescan.PoolScanDriver,
            filescan.PoolScanSymlink,
            filescan.PoolScanMutant,
            filescan.PoolScanProcess,
            modscan.PoolScanModule,
            modscan.PoolScanThread,
            atoms.PoolScanAtom,
            windowstations.PoolScanWind,
            ]

    def calculate(self):
        addr_space = utils.load_as(self._config)

        version = (addr_space.profile.metadata.get("major", 0), 
                   addr_space.profile.metadata.get("minor", 0))

        if version < (6, 0):
            self.scanners.append(sockscan.PoolScanSocket)
            self.scanners.append(connscan.PoolScanConn)
        else:
            self.scanners.append(netscan.PoolScanUdpEndpoint)
            self.scanners.append(netscan.PoolScanTcpListener)
            self.scanners.append(netscan.PoolScanTcpEndpoint)
            self.scanners.append(callbacks.PoolScanDbgPrintCallback)
            self.scanners.append(callbacks.PoolScanRegistryCallback)
            self.scanners.append(callbacks.PoolScanPnp9)
            self.scanners.append(callbacks.PoolScanPnpD)
            self.scanners.append(callbacks.PoolScanPnpC)

        self.scanners.append(callbacks.PoolScanFSCallback)
        self.scanners.append(callbacks.PoolScanShutdownCallback)
        self.scanners.append(callbacks.PoolScanGenericCallback)


        for objct in self.scan_results(addr_space):
            yield objct

    def render_text(self, outfd, data):
        for objct in data:
            print objct


# Volatility
# Copyright (c) 2008-2015 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       The Volatility Foundation
@license:      GNU General Public License 2.0
@contact:      awalters@4tphi.net

This file provides support for Windows 10.
"""

import volatility.plugins.overlays.windows.windows as windows
import volatility.obj as obj
import volatility.win32.tasks as tasks 
import volatility.debug as debug
import volatility.plugins.overlays.windows.win8 as win8

try:
    import distorm3
    has_distorm = True
except ImportError:
    has_distorm = False

class _HMAP_ENTRY(obj.CType):

    @property
    def BlockAddress(self):
        return self.PermanentBinAddress & 0xFFFFFFFFFFF0

class Win10Registry(obj.ProfileModification):
    """The Windows 10 registry HMAP"""

    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 4}

    def modification(self, profile):
        profile.object_classes.update({"_HMAP_ENTRY": _HMAP_ENTRY})

class Win10x64DTB(obj.ProfileModification):
    """The Windows 10 64-bit DTB signature"""

    before = ['WindowsOverlay', 'Windows64Overlay', 'Win8x64DTB']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 4,
                  'memory_model': lambda x: x == '64bit',
                  }

    def modification(self, profile):
        profile.merge_overlay({
            'VOLATILITY_MAGIC': [ None, {
            'DTBSignature' : [ None, ['VolatilityMagic', dict(value = "\x03\x00\xb6\x00")]],
            }]})

class Win10x86DTB(obj.ProfileModification):
    """The Windows 10 32-bit DTB signature"""

    before = ['WindowsOverlay', 'Win8x86DTB']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 4,
                  'memory_model': lambda x: x == '32bit',
                  }

    def modification(self, profile):
        profile.merge_overlay({
            'VOLATILITY_MAGIC': [ None, {
            'DTBSignature' : [ None, ['VolatilityMagic', dict(value = "\x03\x00\x2A\x00")]],
            }]})

class ObHeaderCookieStore(object):
    """A class for finding and storing the nt!ObHeaderCookie value"""

    _instance = None

    def __init__(self):
        self._cookie = None

    def cookie(self):
        return self._cookie 

    def findcookie(self, kernel_space):
        """Find and read the nt!ObHeaderCookie value. 

        On success, return True and save the cookie value in self._cookie.
        On Failure, return False. 

        This method must be called before performing any tasks that require 
        object header validation including handles, psxview (due to pspcid) 
        and the object scanning plugins (psscan, etc). 

        NOTE: this cannot be implemented as a volatility "magic" class,
        because it must be persistent across various classes and sources. 
        We don't want to recalculate the cookie value multiple times. 
        """

        meta = kernel_space.profile.metadata 
        vers = (meta.get("major", 0), meta.get("minor", 0))

        # this algorithm only applies to Windows 10 or greater 
        if vers < (6, 4):
            return True 

        # prevent subsequent attempts from recalculating the existing value 
        if self._cookie:
            return True

        if not has_distorm:
            debug.warning("distorm3 module is not installed")
            return False 

        kdbg = tasks.get_kdbg(kernel_space)
        nt_mod = list(kdbg.modules())[0]

        addr = nt_mod.getprocaddress("ObGetObjectType")
        if addr == None:
            debug.warning("Cannot find nt!ObGetObjectType")
            return False 

        # produce an absolute address by adding the DLL base to the RVA 
        addr += nt_mod.DllBase 
        if not nt_mod.obj_vm.is_valid_address(addr):
            debug.warning("nt!ObGetObjectType at {0} is invalid".format(addr))
            return False 

        # in theory...but so far we haven't tested 32-bits 
        model = meta.get("memory_model")    
        if model == "32bit":
            mode = distorm3.Decode32Bits
        else:
            mode = distorm3.Decode64Bits

        data = nt_mod.obj_vm.read(addr, 100)
        ops = distorm3.Decompose(addr, data, mode, distorm3.DF_STOP_ON_RET)
        addr = None

        # search backwards from the RET and find the MOVZX 

        if model == "32bit":
            # movzx ecx, byte ptr ds:_ObHeaderCookie
            for op in reversed(ops):
                if (op.size == 7 and 
                            'FLAG_DST_WR' in op.flags and
                            len(op.operands) == 2 and 
                            op.operands[0].type == 'Register' and 
                            op.operands[1].type == 'AbsoluteMemoryAddress' and 
                            op.operands[1].size == 8):
                    addr = op.operands[1].disp & 0xFFFFFFFF
                    break
        else:
            # movzx ecx, byte ptr cs:ObHeaderCookie 
            for op in reversed(ops):
                if (op.size == 7 and 
                            'FLAG_RIP_RELATIVE' in op.flags and
                            len(op.operands) == 2 and 
                            op.operands[0].type == 'Register' and 
                            op.operands[1].type == 'AbsoluteMemory' and 
                            op.operands[1].size == 8):
                    addr = op.address + op.size + op.operands[1].disp 
                    break

        if not addr:
            debug.warning("Cannot find nt!ObHeaderCookie")
            return False

        if not nt_mod.obj_vm.is_valid_address(addr):
            debug.warning("nt!ObHeaderCookie at {0} is not valid".format(addr))
            return False

        cookie = obj.Object("unsigned int", offset = addr, vm = nt_mod.obj_vm)
        self._cookie = int(cookie)

        return True

    @staticmethod
    def instance():
        if not ObHeaderCookieStore._instance:
            ObHeaderCookieStore._instance = ObHeaderCookieStore()

        return ObHeaderCookieStore._instance 

class VolatilityCookie(obj.VolatilityMagic):
    """The Windows 10 Cookie Finder"""

    def v(self):
        if self.value is None:
            return self.get_best_suggestion()
        else:
            return self.value

    def get_suggestions(self):
        if self.value:
            yield self.value
        for x in self.generate_suggestions():
            yield x

    def generate_suggestions(self):
        store = ObHeaderCookieStore.instance()
        store.findcookie(self.obj_vm)
        yield store.cookie()

class Win10Cookie(obj.ProfileModification):
    """The Windows 10 Cookie Finder"""

    before = ['WindowsOverlay']

    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 4,
                  }

    def modification(self, profile):
        profile.merge_overlay({
            'VOLATILITY_MAGIC': [ None, {
            'ObHeaderCookie' : [ 0x0, ['VolatilityCookie', dict(configname = "COOKIE")]],
            }]})
        profile.object_classes.update({'VolatilityCookie': VolatilityCookie})

class _OBJECT_HEADER_10(win8._OBJECT_HEADER):
        
    @property
    def TypeIndex(self):
        """Wrap the TypeIndex member with a property that decodes it 
        with the nt!ObHeaderCookie value."""

        cook = obj.VolMagic(self.obj_vm).ObHeaderCookie.v()
        addr = self.obj_offset 
        indx = int(self.m("TypeIndex"))

        return ((addr >> 8) ^ cook ^ indx) & 0xFF

    def is_valid(self):
        """Determine if a given object header is valid"""

        if not obj.CType.is_valid(self):
            return False

        if self.InfoMask > 0x88:
            return False

        if self.PointerCount > 0x1000000 or self.PointerCount < 0:
            return False

        return True

    type_map = {
        2: 'Type',
        3: 'Directory',
        4: 'SymbolicLink',
        5: 'Token',
        6: 'Job',
        7: 'Process',
        8: 'Thread',
        9: 'UserApcReserve',
        10: 'IoCompletionReserve',
        11: 'Silo',
        12: 'DebugObject',
        13: 'Event',
        14: 'Mutant',
        15: 'Callback',
        16: 'Semaphore',
        17: 'Timer',
        18: 'IRTimer',
        19: 'Profile',
        20: 'KeyedEvent',
        21: 'WindowStation',
        22: 'Desktop',
        23: 'Composition',
        24: 'RawInputManager',
        25: 'TpWorkerFactory',
        26: 'Adapter',
        27: 'Controller',
        28: 'Device',
        29: 'Driver',
        30: 'IoCompletion',
        31: 'WaitCompletionPacket',
        32: 'File',
        33: 'TmTm',
        34: 'TmTx',
        35: 'TmRm',
        36: 'TmEn',
        37: 'Section',
        38: 'Session',
        39: 'Partition',
        40: 'Key',
        41: 'ALPC Port',
        42: 'PowerRequest',
        43: 'WmiGuid',
        44: 'EtwRegistration',
        45: 'EtwConsumer',
        46: 'DmaAdapter',
        47: 'DmaDomain',
        48: 'PcwObject',
        49: 'FilterConnectionPort',
        50: 'FilterCommunicationPort',
        51: 'NetworkNamespace',
        52: 'DxgkSharedResource',
        53: 'DxgkSharedSyncObject',
        54: 'DxgkSharedSwapChainObject',
        }

class Win10ObjectHeader(obj.ProfileModification):
    before = ["Win8ObjectClasses"]
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 4}

    def modification(self, profile):
        profile.object_classes.update({"_OBJECT_HEADER": _OBJECT_HEADER_10})

class Win10x64(obj.Profile):
    """ A Profile for Windows 10 x64 """
    _md_memory_model = '64bit'
    _md_os = 'windows'
    _md_major = 6
    _md_minor = 4
    _md_build = 9841
    _md_vtype_module = 'volatility.plugins.overlays.windows.win10_x64_vtypes'

class Win10x86(obj.Profile):
    """ A Profile for Windows 10 x86 """
    _md_memory_model = '32bit'
    _md_os = 'windows'
    _md_major = 6
    _md_minor = 4
    _md_build = 9841
    _md_vtype_module = 'volatility.plugins.overlays.windows.win10_x86_vtypes'



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (C) 2009 Timothy D. Morgan (strings optimization)
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import volatility.win32 as win32
import volatility.debug as debug
import volatility.utils as utils
import volatility.plugins.common as common
import volatility.plugins.taskmods as taskmods
import volatility.plugins.filescan as filescan
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class Strings(common.AbstractWindowsCommand):
    """Match physical offsets to virtual addresses (may take a while, VERY verbose)"""

    def __init__(self, config, *args, **kwargs):
        common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)

        config.add_option('STRING-FILE', short_option = 's', default = None,
                          help = 'File output in strings format (offset:string)',
                          action = 'store', type = 'str')
        config.add_option("SCAN", short_option = 'S', default = False,
                          action = 'store_true', help = 'Use PSScan if no offset is provided')
        config.add_option('OFFSET', short_option = 'o', default = None,
                          help = 'EPROCESS offset (in hex) in the physical address space',
                          action = 'store', type = 'int')
        config.add_option('PID', short_option = 'p', default = None,
                          help = 'Operate on these Process IDs (comma-separated)',
                          action = 'store', type = 'str')      
  
    def get_processes(self, addr_space):
        """Enumerate processes based on user options.

        :param      addr_space | <addrspace.AbstractVirtualAddressSpace>

        :returns    <list> 
        """

        bounce_back = taskmods.DllList.virtual_process_from_physical_offset
        if self._config.OFFSET != None:
            tasks = [bounce_back(addr_space, self._config.OFFSET)]
        elif self._config.SCAN:
            procs = list(filescan.PSScan(self._config).calculate())
            tasks = []
            for task in procs:
                tasks.append(bounce_back(addr_space, task.obj_offset))
        else:
            tasks = win32.tasks.pslist(addr_space)

        try:
            if self._config.PID is not None:
                pidlist = [int(p) for p in self._config.PID.split(',')]
                tasks = [t for t in tasks if int(t.UniqueProcessId) in pidlist]
        except (ValueError, TypeError):
            debug.error("Invalid PID {0}".format(self._config.PID))

        return tasks

    @classmethod
    def get_modules(cls, addr_space):    
        """Enumerate the kernel modules. 

        :param      addr_space | <addrspace.AbstractVirtualAddressSpace>
        
        :returns    <tuple>
        """
        
        modules = win32.modules.lsmod(addr_space)
        mask = addr_space.address_mask
        mods = dict((mask(mod.DllBase), mod) for mod in modules)
        mod_addrs = sorted(mods.keys())
         
        return (mods, mod_addrs)

    @classmethod
    def find_module(cls, mods, mod_addrs, addr_space, vpage):
        """Determine which module owns a virtual page. 

        :param      mods        | <list>
                    mod_addrs   | <list>
                    addr_space  | <addrspace.AbstractVirtualAddressSpace>
                    vpage       | <int> 
        
        :returns    <_LDR_DATA_TABLE_ENTRY> || None
        """
        
        mask = addr_space.address_mask
        return win32.tasks.find_module(mods, mod_addrs, mask(vpage))

    @classmethod
    def get_module_name(cls, module):
        """Get the name of a kernel module.

        :param      module      | <_LDR_DATA_TABLE_ENTRY>
        
        :returns    <str>
        """

        return str(module.BaseDllName or '')

    @classmethod
    def get_task_pid(cls, task):
        """Get the PID of a process. 

        :param      task   | <_EPROCESS>
        
        :returns    <int>
        """

        return task.UniqueProcessId

    def calculate(self):

        if (self._config.STRING_FILE is None or 
                    not os.path.exists(self._config.STRING_FILE)):
            debug.error("Strings file not found")

        addr_space = utils.load_as(self._config)

        layers = [addr_space]
        base = addr_space.base
        while base:
            layers.append(base)
            base = base.base 

        if len(layers) > 2:
            debug.error("Raw memory needed, got {0} (convert with imagecopy)".format(layers[1].__class__.__name__))

        tasks = self.get_processes(addr_space)

        stringlist = open(self._config.STRING_FILE, "r")
        reverse_map = self.get_reverse_map(addr_space, tasks)

        for line in stringlist:
            try:
                (offsetString, string) = self.parse_line(line)
                offset = int(offsetString)
            except ValueError:
                debug.error("String file format invalid.")

            pids = ["FREE MEMORY:-1"]
            if reverse_map.has_key(offset & 0xFFFFFFFFFFFFF000):
                pids = ["{0}:{1:08x}".format(pid[0], pid[1] | (offset & 0xFFF)) for pid in reverse_map[offset & 0xFFFFFFFFFFFFF000][1:]]

            yield offset, pids, "{0}".format(string.strip())

    @classmethod
    def parse_line(cls, line):
        """Parses a line of strings. 

        :param      cls     | <Strings>
                    line    | <str>
        
        :returns    <tuple>
        """
        # Remove any leading spaces to handle nasty strings output
        line = line.lstrip()
        maxlen = len(line)
        split_char = ' '
        for char in [' ', ':']:
            charpos = line.find(char)
            if charpos < maxlen and charpos > 0:
                split_char = char
                maxlen = charpos
        return tuple(line.split(split_char, 1))

    @classmethod
    def get_reverse_map(cls, addr_space, tasks):
        """Generates a reverse mapping of physical addresses 
        to the kernel and/or tasks.

        :param      addr_space  | <addrspace.AbstractVirtualAddressSpace>
                    tasks       | <list> 
    
        :returns    <dict>
        """

        # ASSUMPTION: no pages mapped in kernel and userland
        # XXX: Can we eliminate the above assumption?  It seems like the only change needed for
        #      that would be to store a boolean with each pid/vaddr pair...
        #
        # XXX: The following code still fails to represent information about larger pages in
        #      the final output.  The output implies that addresses in a large page are
        #      really stored in one or more 4k pages.  This is no different from the old
        #      version of the code, but in this version it could be corrected easily by
        #      recording vpage instead of vpage+i in the reverse map. -- TDM
        reverse_map = {}

        (mods, mod_addrs) = cls.get_modules(addr_space)
   
        debug.debug("Calculating kernel mapping...\n")
        available_pages = addr_space.get_available_pages()
        for (vpage, vpage_size) in available_pages:
            kpage = addr_space.vtop(vpage)
            for i in range(0, vpage_size, 0x1000):
                # Since the output will always be mutable, we 
                # don't need to reinsert into the list
                pagelist = reverse_map.get(kpage + i, None)
                if pagelist is None:
                    pagelist = [True]
                    reverse_map[kpage + i] = pagelist
                # Try to lookup the owning kernel module
                module = cls.find_module(mods, mod_addrs, addr_space, vpage + i)
                if module:
                    hint = cls.get_module_name(module)
                else:
                    hint = 'kernel'
                pagelist.append((hint, vpage + i))

        debug.debug("Calculating task mappings...\n")
        for task in tasks:
            task_space = task.get_process_address_space()
            debug.debug("  Task {0} ...".format(cls.get_task_pid(task)))
            process_id = cls.get_task_pid(task)
            try:
                available_pages = task_space.get_available_pages()
                for (vpage, vpage_size) in available_pages:
                    physpage = task_space.vtop(vpage)
                    for i in range(0, vpage_size, 0x1000):
                        # Since the output will always be mutable, we 
                        # don't need to reinsert into the list
                        pagelist = reverse_map.get(physpage + i, None)
                        if pagelist is None:
                            pagelist = [False]
                            reverse_map[physpage + i] = pagelist
                        if not pagelist[0]:
                            pagelist.append((process_id, vpage + i))

            except (AttributeError, ValueError, TypeError):
                # Handle most errors, but not all of them
                continue
        
        return reverse_map

    def unified_output(self, data):
        return TreeGrid([("Offset(P)", Address),
                       ("Attribution", str),
                       ("Offset(V)", Address),
                       ("String", str)],
                        self.generator(data))

    def generator(self, data):
        for offset, pids, string in data:
            for p in pids:
                item, addr = p.split(":")
                yield (0, [Address(offset),
                        str(item),
                        Address(int(addr, 16)),
                        str(string)])

    def render_text(self, outfd, data):
        for offset, pids, string in data:
            outfd.write("{0} [{1}] {2}\n".format(offset, ' '.join(pids), string))




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import os
import volatility.obj as obj
import volatility.debug as debug
import volatility.plugins.mac.pstasks as mac_tasks
import volatility.plugins.mac.common  as mac_common
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class mac_procdump(mac_tasks.mac_tasks):
    """ Dumps the executable of a process """

    def __init__(self, config, *args, **kwargs):         
        mac_tasks.mac_tasks.__init__(self, config, *args, **kwargs)         
        self._config.add_option('DUMP-DIR', short_option = 'D', default = None, help = 'Output directory', action = 'store', type = 'str')

    def unified_output(self, data):
        if (not self._config.DUMP_DIR or not os.path.isdir(self._config.DUMP_DIR)):
            debug.error("Please specify an existing output dir (--dump-dir)")
 
        return TreeGrid([("Task", str),
                        ("Pid", int),
                        ("Address", Address),
                        ("Path", str),
                        ], self.generator(data))

    def generator(self, data):
        for proc in data:
            exe_address = proc.text_start()

            if exe_address:
                file_path = mac_common.write_macho_file(self._config.DUMP_DIR, proc, exe_address)
                yield (0, [
                    str(proc.p_comm),
                    int(proc.p_pid),
                    Address(exe_address),
                    str(file_path),
                    ])

    def render_text(self, outfd, data):
        if (not self._config.DUMP_DIR or not os.path.isdir(self._config.DUMP_DIR)):
            debug.error("Please specify an existing output dir (--dump-dir)")
 
        self.table_header(outfd, [("Task", "25"), 
                                  ("Pid", "6"),
                                  ("Address", "[addrpad]"),
                                  ("Path", "")])
       
        for proc in data:
            exe_address = proc.text_start()
            if exe_address:
                file_path = mac_common.write_macho_file(self._config.DUMP_DIR, proc, exe_address)
                self.table_row(outfd, proc.p_comm, proc.p_pid, exe_address, file_path)




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (C) 2010,2011,2012 Michael Hale Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#
from volatility import renderers
from volatility.renderers.basic import Address
from volatility.renderers.text import TextRenderer

import volatility.utils as utils
import volatility.debug as debug
import volatility.plugins.gui.constants as consts
import volatility.plugins.gui.sessions as sessions

class Gahti(sessions.Sessions):
    """Dump the USER handle type information"""

    def unified_output(self, data):

        return renderers.TreeGrid(
            [("Session", str),
             ("Type", str),
             ("Tag", str),
             ("fnDestroy", Address),
             ("Flags", str),
            ], self.generator(data))

    def generator(self, data):
        profile = utils.load_as(self._config).profile

        # Get the OS version being analyzed
        version = (profile.metadata.get('major', 0),
                   profile.metadata.get('minor', 0))

        # Choose which USER handle enum to use
        if version >= (6, 1):
            handle_types = consts.HANDLE_TYPE_ENUM_SEVEN
        else:
            handle_types = consts.HANDLE_TYPE_ENUM

        for session in data:
            gahti = session.find_gahti()
            if gahti:
                for i, h in handle_types.items():
                    yield (0,
                                    [str(session.SessionId),
                                     str(h),
                                     str(gahti.types[i].dwAllocTag),
                                     Address(gahti.types[i].fnDestroy),
                                     str(gahti.types[i].bObjectCreateFlags)])

    def render_text(self, outfd, data):
        output = self.unified_output(data)

        if isinstance(output, renderers.TreeGrid):
            tr = TextRenderer(self.text_cell_renderers, sort_column = self.text_sort_column)
            tr.render(outfd, output)
        else:
            raise TypeError("Unified Output must return a TreeGrid object")



# Volatility
# Copyright (C) 2008-2013 Volatility Foundation
# Copyright (C) 2011 Jamie Levy (Gleeda) <jamie@memoryanalysis.net>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Jamie Levy (gleeda)
@license:      GNU General Public License 2.0
@contact:      jamie@memoryanalysis.net
@organization: Volatility Foundation
"""

import volatility.plugins.registry.registryapi as registryapi
from volatility.renderers import TreeGrid
import volatility.debug as debug
import volatility.utils as utils
import volatility.obj as obj
import volatility.plugins.common as common
import volatility.addrspace as addrspace

# Structures taken from the ShimCache Whitepaper: https://blog.mandiant.com/archives/2459

#### SHIMRECS ####

shimrecs_type_xp = {
    'ShimRecords' : [ None, {
        'Magic' : [ 0x0, ['unsigned int']], #0xDEADBEEF
        'NumRecords' : [ 0x8, ['short']],
        'Entries' : [0x190, ['array', lambda x: x.NumRecords, ['AppCompatCacheEntry']]],
    } ],
}

shimrecs_type_2003vista = {
    'ShimRecords' : [ None, {
        'Magic' : [ 0x0, ['unsigned int']], #0xBADC0FFE
        'NumRecords' : [ 0x4, ['int']],
        'Entries' : [0x8, ['array', lambda x: x.NumRecords, ['AppCompatCacheEntry']]],
    } ],
}

shimrecs_type_win7 = {
    'ShimRecords' : [ None, {
        'Magic' : [ 0x0, ['unsigned int']], #0xBADC0FEE
        'NumRecords' : [ 0x4, ['int']],
        'Entries' : [0x80, ['array', lambda x: x.NumRecords, ['AppCompatCacheEntry']]],
    } ],
}

#### APPCOMPAT TYPES ####

appcompat_type_xp_x86 = {
    'AppCompatCacheEntry' : [ 0x228, {
        'Path' : [ 0x0, ['NullString', dict(length = 0x208, encoding = 'utf8')]],
        'LastModified' : [ 0x210, ['WinTimeStamp', dict(is_utc = True)]],
        'FileSize': [0x218, ['long long']],
        'LastUpdate' : [ 0x220, ['WinTimeStamp', dict(is_utc = True)]],
    } ],
}

appcompat_type_2003_x86 = {
    'AppCompatCacheEntry' : [ 0x18, {
        'Length' : [ 0x0, ['unsigned short']],
        'MaximumLength' : [0x2, ['unsigned short']],
        'PathOffset' : [ 0x4, ['unsigned int']],
        'LastModified' : [ 0x8, ['WinTimeStamp', dict(is_utc = True)]],
        'FileSize': [0x10, ['_LARGE_INTEGER']],
    } ],
}

appcompat_type_vista_x86 = {
    'AppCompatCacheEntry' : [ 0x18, {
        'Length' : [ 0x0, ['unsigned short']],
        'MaximumLength' : [0x2, ['unsigned short']],
        'PathOffset' : [ 0x4, ['unsigned int']],
        'LastModified' : [ 0x8, ['WinTimeStamp', dict(is_utc = True)]],
        'InsertFlags' : [0x10, ['unsigned int']],
        'Flags' : [0x14, ['unsigned int']],
    } ],
}

appcompat_type_win7_x86 = {
    'AppCompatCacheEntry' : [ 0x20, {
        'Length' : [ 0x0, ['unsigned short']],
        'MaximumLength' : [0x2, ['unsigned short']],
        'PathOffset' : [ 0x4, ['unsigned int']],
        'LastModified' : [ 0x8, ['WinTimeStamp', dict(is_utc = True)]],
        'InsertFlags' : [0x10, ['unsigned int']],
        'ShimFlags' : [0x14, ['unsigned int']],
        'BlobSize' : [0x18, ['unsigned int']],
        'BlobOffset' : [0x1c, ['unsigned int']],
    } ],
}

appcompat_type_2003_x64 = {
    'AppCompatCacheEntry' : [ 0x20, {
        'Length' : [ 0x0, ['unsigned short']],
        'MaximumLength' : [0x2, ['unsigned short']],
        'PathOffset' : [ 0x8, ['unsigned long long']],
        'LastModified' : [ 0x10, ['WinTimeStamp', dict(is_utc = True)]],
        'FileSize': [0x18, ['_LARGE_INTEGER']],
    } ],
}

appcompat_type_vista_x64 = {
    'AppCompatCacheEntry' : [ 0x20, {
        'Length' : [ 0x0, ['unsigned short']],
        'MaximumLength' : [0x2, ['unsigned short']],
        'PathOffset' : [ 0x8, ['unsigned int']],
        'LastModified' : [ 0x10, ['WinTimeStamp', dict(is_utc = True)]],
        'InsertFlags' : [0x18, ['unsigned int']],
        'Flags' : [0x1c, ['unsigned int']],
    } ],
}

appcompat_type_win7_x64 = {
    'AppCompatCacheEntry' : [ 0x30, {
        'Length' : [ 0x0, ['unsigned short']],
        'MaximumLength' : [0x2, ['unsigned short']],
        'PathOffset' : [ 0x8, ['unsigned long long']],
        'LastModified' : [ 0x10, ['WinTimeStamp', dict(is_utc = True)]],
        'InsertFlags' : [0x18, ['unsigned int']],
        'ShimFlags' : [0x1c, ['unsigned int']],
        'BlobSize' : [0x20, ['unsigned long long']],
        'BlobOffset' : [0x28, ['unsigned long long']],
    } ],
}

class ShimCacheTypesXPx86(obj.ProfileModification):
    before = ['WindowsObjectClasses']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 5,
                  'minor': lambda x: x == 1,
                  'memory_model': lambda x: x == '32bit'}
    def modification(self, profile):
        profile.vtypes.update(shimrecs_type_xp)
        profile.vtypes.update(appcompat_type_xp_x86)


class ShimCacheTypes2003x86(obj.ProfileModification):
    before = ['WindowsObjectClasses']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 5,
                  'minor': lambda x: x == 2,
                  'memory_model': lambda x: x == '32bit'}
    def modification(self, profile):
        profile.vtypes.update(shimrecs_type_2003vista)
        profile.vtypes.update(appcompat_type_2003_x86)

class ShimCacheTypesVistax86(obj.ProfileModification):
    before = ['WindowsObjectClasses']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 0,
                  'memory_model': lambda x: x == '32bit'}
    def modification(self, profile):
        profile.vtypes.update(shimrecs_type_2003vista)
        profile.vtypes.update(appcompat_type_vista_x86)

class ShimCacheTypesWin7x86(obj.ProfileModification):
    before = ['WindowsObjectClasses']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 1,
                  'memory_model': lambda x: x == '32bit'}
    def modification(self, profile):
        profile.vtypes.update(shimrecs_type_win7)
        profile.vtypes.update(appcompat_type_win7_x86)

class ShimCacheTypes2003x64(obj.ProfileModification):
    before = ['WindowsObjectClasses']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 5,
                  'minor': lambda x: x == 2,
                  'memory_model': lambda x: x == '64bit'}
    def modification(self, profile):
        profile.vtypes.update(shimrecs_type_2003vista)
        profile.vtypes.update(appcompat_type_2003_x64)

class ShimCacheTypesVistax64(obj.ProfileModification):
    before = ['WindowsObjectClasses']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 0,
                  'memory_model': lambda x: x == '64bit'}
    def modification(self, profile):
        profile.vtypes.update(shimrecs_type_2003vista)
        profile.vtypes.update(appcompat_type_vista_x64)

class ShimCacheTypesWin7x64(obj.ProfileModification):
    before = ['WindowsObjectClasses']
    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 1,
                  'memory_model': lambda x: x == '64bit'}
    def modification(self, profile):
        profile.vtypes.update(shimrecs_type_win7)
        profile.vtypes.update(appcompat_type_win7_x64)


class ShimCache(common.AbstractWindowsCommand):
    """Parses the Application Compatibility Shim Cache registry key"""
    def __init__(self, config, *args, **kwargs):
        self._addrspace = None
        common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)

    @staticmethod
    def is_valid_profile(profile):
        return profile.metadata.get('os', 'unknown').lower() == 'windows'

    @staticmethod
    def remove_unprintable(item):
        return ''.join([str(c) for c in item if (ord(c) > 31 or ord(c) == 9) and ord(c) <= 126])

    @staticmethod
    def get_entries(addr_space, regapi):

        regapi.reset_current()
        currentcs = regapi.reg_get_currentcontrolset()
        if currentcs == None:
            currentcs = "ControlSet001"

        version = (addr_space.profile.metadata.get('major', 0),
                   addr_space.profile.metadata.get('minor', 0))
        xp = False

        if version <= (5, 1):
            key = currentcs + "\\Control\\Session Manager\\AppCompatibility"
            xp = True
        else:
            key = currentcs + "\\Control\\Session Manager\\AppCompatCache"

        data_raw = regapi.reg_get_value('system', key, "AppCompatCache")
        if data_raw == None or len(data_raw) < 0x1c:
            debug.warning("No ShimCache data found")
            raise StopIteration

        bufferas = addrspace.BufferAddressSpace(addr_space.get_config(), data = data_raw)
        shimdata = obj.Object("ShimRecords", offset = 0, vm = bufferas)
        if shimdata == None:
            debug.warning("No ShimCache data found")
            raise StopIteration

        if shimdata.Magic not in [0xDEADBEEF, 0xBADC0FFE, 0xBADC0FEE]:
            debug.warning("ShimRecords.Magic value {0:X} is not valid".format(shimdata.Magic))
            raise StopIteration

        for e in shimdata.Entries:
            if xp:
                yield e.Path, e.LastModified, e.LastUpdate
            else:
                yield ShimCache.remove_unprintable(bufferas.read(int(e.PathOffset), int(e.Length))), e.LastModified, None

    def calculate(self):
        addr_space = utils.load_as(self._config)
        regapi = registryapi.RegistryApi(self._config)

        for entry in self.get_entries(addr_space, regapi):
            yield entry

    def unified_output(self, data):
        # blank header in case there is no shimcache data
        return TreeGrid([("Last Modified", str),
                       ("Last Update", str),
                       ("Path", str),
                      ], self.generator(data))

    def generator(self, data):
        for path, lm, lu in data:
            if lu:
                yield (0, [str(lm), str(lu), str(path).strip()])
            else:
                yield (0, [str(lm), "-", str(path).strip()])

    def render_text(self, outfd, data):
        first = True
        for path, lm, lu in data:
            if lu:
                if first:
                    self.table_header(outfd, [("Last Modified", "30"),
                                              ("Last Update", "30"),
                                              ("Path", ""),
                                             ])
                    first = False
                outfd.write("{0:30} {1:30} {2}\n".format(lm, lu, path))
            else:
                if first:
                    self.table_header(outfd, [("Last Modified", "30"),
                                              ("Path", ""),
                                             ])
                    first = False
                outfd.write("{0:30} {1}\n".format(lm, path))



# 
# Python port of WKdm compression  / decompression by 
# Golden G. Richard III (@nolaforensix, golden@arcanealloy.com)
# December 2013
#
# For compression and decompression of 4K pages.
#
# Based loosely on WKdm.c, by:
# 
# *  Paul Wilson -- wilson@cs.utexas.edu
# *  Scott F. Kaplan -- sfkaplan@cs.utexas.edu
# *  September 1997
#
# but designed specifically to be compatible with the optimized x86_64
# assembler version in xnu-2422.1.72/osfmk/x86_64/WKdmCompress_new*
# (Mac OS X Mavericks 10.9 kernel source).  Apples's assembler version
# eliminates the unused "version word" in the header, reducing header
# size to 3 words, making this version incompatible with the original
# WKdm.c (for what it's worth).  The Apple version also introduces a
# compression budget for WKdm_compress, which results in a compression
# failure if the budget (expressed in bytes) is exceeded.  The
# compression budget is also supported by this version for
# compatibility.
#

import math

class WKdm:
    
    ##################################################################
    ##################################################################
    # DO NOT CHANGE THESE: Correct operation depends on 4K page size
    # and there are various other non-trivial dependencies
    ##################################################################
    ##################################################################
    WORD_SIZE_IN_BYTES        = 4
    PAGE_SIZE_IN_WORDS	      = 1024
    PAGE_SIZE_IN_BYTES	      = 4096
    DICTIONARY_SIZE_IN_WORDS  = 16
    HEADER_SIZE_IN_WORDS      = 3                      
    TAGS_AREA_OFFSET_IN_WORDS = HEADER_SIZE_IN_WORDS
    TAGS_AREA_SIZE_IN_WORDS   = 64
    NUM_LOW_BITS	      = 10
    LOW_BITS_MASK	      = 0x3FF
    ALL_ONES_MASK	      = 0xFFFFFFFF
    TWO_BITS_PACKING_MASK     = 0x03030303
    FOUR_BITS_PACKING_MASK    = 0x0F0F0F0F
    TEN_LOW_BITS_MASK	      = 0x000003FF
    TWENTY_TWO_HIGH_BITS_MASK = 0xFFFFFC00
    ZERO_TAG	              = 0x0
    PARTIAL_TAG	              = 0x1
    MISS_TAG	              = 0x2
    EXACT_TAG	              = 0x3
    SINGLE_BYTE_MASKS = [0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000]
    ##################################################################
    ##################################################################

    ##################################################################
    # These are the constants for the hash function lookup table.
    # Only zero maps to zero.  The rest of the table is the result of
    # appending 17 randomizations of the multiples of 4 from 4 to 56.
    ##################################################################
    HASH_LOOKUP_TABLE_CONTENTS = [ 
        0, 52,  8, 56, 16, 12, 28, 20,  4, 36, 48, 24, 44, 40, 32, 60, 
        8, 12, 28, 20,  4, 60, 16, 36, 24, 48, 44, 32, 52, 56, 40, 12, 
        8, 48, 16, 52, 60, 28, 56, 32, 20, 24, 36, 40, 44,  4,  8, 40, 
        60, 32, 20, 44,  4, 36, 52, 24, 16, 56, 48, 12, 28, 16,  8, 40, 
        36, 28, 32, 12,  4, 44, 52, 20, 24, 48, 60, 56, 40, 48,  8, 32, 
        28, 36,  4, 44, 20, 56, 60, 24, 52, 16, 12, 12,  4, 48, 20,  8, 
        52, 16, 60, 24, 36, 44, 28, 56, 40, 32, 36, 20, 24, 60, 40, 44, 
        52, 16, 32,  4, 48,  8, 28, 56, 12, 28, 32, 40, 52, 36, 16, 20, 
        48,  8,  4, 60, 24, 56, 44, 12,  8, 36, 24, 28, 16, 60, 20, 56, 
        32, 40, 48, 12,  4, 44, 52, 44, 40, 12, 56,  8, 36, 24, 60, 28, 
        48,  4, 32, 20, 16, 52, 60, 12, 24, 36,  8,  4, 16, 56, 48, 44, 
        40, 52, 32, 20, 28, 32, 12, 36, 28, 24, 56, 40, 16, 52, 44,  4, 
        20, 60,  8, 48, 48, 52, 12, 20, 32, 44, 36, 28,  4, 40, 24,  8, 
        56, 60, 16, 36, 32,  8, 40,  4, 52, 24, 44, 20, 12, 28, 48, 56, 
        16, 60,  4, 52, 60, 48, 20, 16, 56, 44, 24,  8, 40, 12, 32, 28, 
        36, 24, 32, 12,  4, 20, 16, 60, 36, 28,  8, 52, 40, 48, 44, 56  
    ]


    ##################################################################
    # WK_pack_2bits(): Pack some multiple of four words holding
    # two-bit tags (in the low two bits of each byte) into an integral
    # number of words, i.e., one fourth as many. Data in the
    # source_buf is used starting at index 0 up to and not including
    # index source_end. The packed data is written into the dest_buf
    # starting at index dest_start.  NOTE: Pad the input with zeroes
    # to a multiple of four words, or else.
    ##################################################################
    def WK_pack_2bits(self, 
                      source_buf,
                      source_end,
                      dest_buf,
                      dest_start):

        j = dest_start
        k = source_end

        src_next = 0
        
        # loop to repeatedly grab four input words and pack it into 1
        # output word.
        while src_next < source_end:
            temp = source_buf[src_next]
            temp |= (source_buf[src_next+1] << 2)
            temp |= (source_buf[src_next+2] << 4)
            temp |= (source_buf[src_next+3] << 6)
            dest_buf[dest_start] = temp

            dest_start += 1     
            src_next += 4

        return dest_start

        
    ###################################################################
    # WK_pack_4bits(): Pack an even number of words holding 4-bit
    # patterns in the low bits of each byte into half as many
    # words. Data in the source_buf is used starting at index 0 up
    # to and not including index source_end. The packed data is
    # written into the dest_buf starting at index dest_start.
    # NOTE: Pad the input with zeroes to an even number of words,
    # or else.
    ################################################################## 
    def WK_pack_4bits(self, 
                      source_buf,
                      source_end,
                      dest_buf, 
                      dest_start):
        src_next = 0
  
        # loop to repeatedly grab two input words and pack it into 1
        # output word.
        while src_next < source_end:
            temp = source_buf[src_next]
            temp |= (source_buf[src_next+1] << 4)
            dest_buf[dest_start] = temp
            
            dest_start += 1     
            src_next += 2

        return dest_start

    
    ###################################################################
    # WK_pack_3_tenbits(): Pack a sequence of three ten bit items
    # into one word. Data in the source_buf is used starting at
    # index 0 up to and not including source_end. The packed data
    # is written into the dest_buf starting at index dest_start.
    # NOTE: Pad out the input with zeroes to an even number of
    # words, or else.
    ################################################################### 
    def WK_pack_3_tenbits(self,
                          source_buf,
                          source_end,
                          dest_buf,
                          dest_start):
        
        src_next = 0
            
        # loop to repeatedly grab three input words and pack it into 1
        # output word.
        while src_next < source_end:
            temp = source_buf[src_next]
            temp |= (source_buf[src_next+1] << 10)
            temp |= (source_buf[src_next+2] << 20)
            dest_buf[dest_start] = temp
            
            dest_start += 1     
            src_next += 3
            
        return dest_start

    
     ################################################################### 
     # WK_unpack_2bits(): Take any number of words containing 16
     # two-bit values and unpack them into four times as many words
     # containg those two bit values as bytes (with the low two
     # bits of each byte holding the actual value).  Data is read
     # from input_buf starting at index input_start and up to but
     # not including input_end.  Unpacked data is placed in
     # output_buf.
     ################################################################### 
    def WK_unpack_2bits(self,
                        input_buf,
                        input_start,
                        input_end,
                        output_buf):
        
        output_next = 0
            
        # loop to repeatedly grab one input word and unpack it into
        # 4 output words.
        while input_start < input_end:
            temp = input_buf[input_start]
            output_buf[output_next]   = temp        & self.TWO_BITS_PACKING_MASK
            output_buf[output_next+1] = (temp >> 2) & self.TWO_BITS_PACKING_MASK
            output_buf[output_next+2] = (temp >> 4) & self.TWO_BITS_PACKING_MASK
            output_buf[output_next+3] = (temp >> 6) & self.TWO_BITS_PACKING_MASK
            
            output_next += 4
            input_start += 1
            
        return output_next


    ################################################################### 
    # WK_unpack_4bits(): Unpack four bits consumes any number of
    # words holding 8 4-bit values per word, and unpacks them into
    # twice as many words, with each value in a separate byte.
    # (The four-bit values occupy the low halves of the bytes in
    # the result). Data is read from input_buf starting at index
    # input_start and up to but not including input_end.  Unpacked
    # data is placed in output_buf.
    ################################################################### 
    def WK_unpack_4bits(self,
                        input_buf,
                        input_start,
                        input_end,
                        output_buf):

        output_next = 0

        # loop to repeatedly grab one input word and unpack it into 2
        # output words.
        while input_start < input_end:
            temp = input_buf[input_start]
            output_buf[output_next] = temp & self.FOUR_BITS_PACKING_MASK
            output_buf[output_next+1] = (temp >> 4) & self.FOUR_BITS_PACKING_MASK        

            output_next += 2
            input_start += 1

        return output_next

        
    ################################################################### 
    # WK_unpack_3_tenbits(): Unpack three 10-bit items from the
    # low 30 bits of any number of 32-bit words. Data is read from
    # input_buf starting at index input_start and up to but not
    # including input_end.  Unpacked data is placed in output_buf.
    ################################################################### 
    def WK_unpack_3_tenbits(self,
                            input_buf,
                            input_start,
                            input_end,
                            output_buf):

        output_next = 0

        # loop to fetch 1 word of input, splitting each into three words of
        # output with 10 meaningful low order bits.
        while input_start < input_end:
            temp = input_buf[input_start]
            output_buf[output_next] = temp & self.LOW_BITS_MASK
            output_buf[output_next+1] = (temp >> 10) & self.LOW_BITS_MASK
            output_buf[output_next+2] = temp >> 20

            input_start += 1
            output_next += 3

        return output_next


    ################################################################### 
    # WKdm_compress(): Compress a src_buf containing num_input_words
    # 32-bit words into a dest_buf of 32-bit words.  Returns size of
    # dest_buf or -1 if the compression budget (expressed in bytes) is
    # exceeeded, which also results in undefined contents in dest_buf.
    ################################################################### 
    def WKdm_compress(self, 
                      src_buf,
                      dest_buf,
                      num_input_words,
                      compression_budget):

        dictionary=[0] * self.DICTIONARY_SIZE_IN_WORDS
        hashLookupTable = self.HASH_LOOKUP_TABLE_CONTENTS

        # update compression budget based on fixed overhead
        compression_budget -= (self.HEADER_SIZE_IN_WORDS + 
                               self.TAGS_AREA_SIZE_IN_WORDS) * self.WORD_SIZE_IN_BYTES

        # arrays that hold output data in intermediate form during modeling 
        # and whose contents are packed into the actual output after modeling 

        tempTagsArray = [0] * 300                # tags for everything          
        tempQPosArray = [0] * 300                # queue positions for matches  
        tempLowBitsArray = [0] * 1200            # low bits for partial matches 

        # boundary_tmp will be used for keeping track of what's where in
        # the compressed page during packing

        boundary_tmp=0

        next_full_patt = 0                   # index into dest_buf
        next_tag = 0                         # index into tempTagsArray 
        next_qp = 0                          # index into tempQPosArray
        next_low_bits = 0                    # index into tempLowBitsArray
        next_input_word = 0                  # index into src_buf

        # initialize dictionary
        for i in range(0,15):
            dictionary[i] = 1

        # process all input words
        next_full_patt = self.TAGS_AREA_OFFSET_IN_WORDS + self.TAGS_AREA_SIZE_IN_WORDS 
        while next_input_word < num_input_words:
            input_word = src_buf[next_input_word]
            dict_location = hashLookupTable[(input_word >> 10) & 0xFF] / 4
            dict_word = dictionary[dict_location]

            if input_word == dict_word:
                tempTagsArray[next_tag / 4] |= (self.EXACT_TAG << (((next_tag) % 4) * 8))
                next_tag += 1
                tempQPosArray[next_qp / 4] |= (dict_location << (((next_qp) % 4) * 8))
                next_qp += 1
            elif input_word == 0:
                tempTagsArray[next_tag / 4] |= (self.ZERO_TAG << (((next_tag) % 4) * 8))
                next_tag += 1
            else:
                input_high_bits = input_word >> self.NUM_LOW_BITS
                dict_word_high_bits = dict_word >> self.NUM_LOW_BITS
                if input_high_bits == dict_word_high_bits:
                    tempTagsArray[next_tag / 4] |= (self.PARTIAL_TAG << (((next_tag) % 4) * 8))
                    next_tag += 1
                    tempQPosArray[next_qp / 4] |= (dict_location << (((next_qp) % 4) * 8))
                    next_qp += 1
                    tempLowBitsArray[next_low_bits] = input_word & self.LOW_BITS_MASK
                    next_low_bits += 1
                    dictionary[dict_location] = input_word
                else:
                    # check compression budget and fail immediately if exhausted
                    compression_budget -= self.WORD_SIZE_IN_BYTES
                    if compression_budget < 0:
                        return -1

                    tempTagsArray[next_tag / 4] |= (self.MISS_TAG << (((next_tag) % 4) * 8))
                    next_tag += 1
                    dest_buf[next_full_patt] = input_word
                    next_full_patt += 1
                    dictionary[dict_location] = input_word

            next_input_word += 1

        dest_buf[0] = next_full_patt            # qpos area start

        # Pack the tags into the tags area, between the page header
        # and the full words area.  No padding because page size is
        # assumed to be a multiple of 16.  Compression budget associated
        # with this area has already been deducted.

        boundary_tmp = self.WK_pack_2bits(tempTagsArray,
                                          next_tag / 4,                                         
                                          dest_buf,
                                          self.TAGS_AREA_OFFSET_IN_WORDS)

        # Pack the queue positions into the area just after the full
        # words.  Round up the size of the region to a multiple of two
        # words.

        endQPosArray = int(math.ceil(next_qp / 8.0)) * 2
        next_qp = int(math.ceil(next_qp / 4.0))

        # Pad the array with zeros to avoid corrupting real packed
        # values. 
        
        while (next_qp < endQPosArray):
            tempQPosArray[next_qp] = 0
            next_qp += 1


        # check compression budget and fail immediately if exhausted
        compression_budget -= (endQPosArray / 2) * self.WORD_SIZE_IN_BYTES
        if compression_budget < 0:
            return -1

        boundary_tmp = self.WK_pack_4bits(tempQPosArray,
                                          endQPosArray,
                                          dest_buf,
                                          next_full_patt)
        
        # Record (in the header) where packing queue positions stopped,
        # which is where packing of low bits will start.

        dest_buf[1] = boundary_tmp

        # Pack the low bit patterns into the area just after the queue
        # positions.  Round up the size of the region region to a
        # multiple of three words.

        endLowBitsArray = int(math.ceil(next_low_bits / 3.0)) * 3

        # Pad the array with zeros to avoid corrupting real packed
        # values. 

        while (next_low_bits < endLowBitsArray):
            tempLowBitsArray[next_low_bits] = 0
            next_low_bits += 1
            
        # check compression budget and fail immediately if exhausted
        compression_budget -= (endLowBitsArray / 3) * self.WORD_SIZE_IN_BYTES
        if compression_budget < 0:
            return -1

        boundary_tmp = self.WK_pack_3_tenbits (tempLowBitsArray,
                                               endLowBitsArray,
                                               dest_buf,
                                               boundary_tmp)

        dest_buf[2] = boundary_tmp

        return boundary_tmp


    ################################################################### 
    # WKdm_decompress(): Decompress a src_buf containing 32-bit words
    # into a dest_buf of 32-bit words.  Returns size of decompressed
    # buffer or -1 on decompression error (in which case the
    # dest_buf contents are undefined).
    ###################################################################  
    def WKdm_decompress (self,
                         src_buf,
                         dest_buf):

        dictionary = [0] * self.DICTIONARY_SIZE_IN_WORDS
        hashLookupTable = self.HASH_LOOKUP_TABLE_CONTENTS

        # arrays that hold output data in intermediate form during modeling 
        # and whose contents are packed into the actual output after modeling 

        tempTagsArray = [0] * 300        # tags for everything          
        tempQPosArray = [0] * 300        # queue positions for matches  
        tempLowBitsArray = [0] * 1200    # low bits for partial matches 

        # initialize dictionary

        for i in range(0,15):
            dictionary[i] = 1

        try:
            self.WK_unpack_2bits(src_buf,
                                 self.TAGS_AREA_OFFSET_IN_WORDS,
                                 self.TAGS_AREA_OFFSET_IN_WORDS + self.TAGS_AREA_SIZE_IN_WORDS,
                                 tempTagsArray)

            self.WK_unpack_4bits(src_buf,
                                 src_buf[0], 
                                 src_buf[1],
                                 tempQPosArray)

            self.WK_unpack_3_tenbits(src_buf,
                                     src_buf[1],
                                     src_buf[2], 
                                     tempLowBitsArray)

            next_tag = 0                                                                     # index into tempTagsArray
            tags_area_end = self.PAGE_SIZE_IN_WORDS 
            next_qp = 0                                                                      # index into tempQPosArray
            next_low_bits = 0                                                                # index into tempLowBitsArray
            next_full_word = self.TAGS_AREA_OFFSET_IN_WORDS + self.TAGS_AREA_SIZE_IN_WORDS   # index into src_buf
            next_output = 0                                                                  # index into dest_buf

            while (next_tag < tags_area_end):
                tag = (tempTagsArray[next_tag / 4] & self.SINGLE_BYTE_MASKS[next_tag % 4]) >> (((next_tag) % 4) * 8)

                if tag == self.ZERO_TAG:
                    dest_buf[next_output] = 0
                elif tag == self.EXACT_TAG:
                    dict_location = (tempQPosArray[next_qp / 4] & self.SINGLE_BYTE_MASKS[next_qp % 4]) >> (((next_qp) % 4) * 8)
                    next_qp += 1
                    dest_buf[next_output] = dictionary[dict_location]
                elif tag == self.PARTIAL_TAG:
                    dict_location = (tempQPosArray[next_qp / 4] & self.SINGLE_BYTE_MASKS[next_qp % 4]) >> (((next_qp) % 4) * 8)
                    temp = dictionary[dict_location]
                    # strip out low bits 
                    temp = ((temp >> self.NUM_LOW_BITS) << self.NUM_LOW_BITS)
                    # add in stored low bits from temp array 
                    temp = temp | tempLowBitsArray[next_low_bits]
                    next_low_bits += 1
                    # replace old value in dict
                    dictionary[dict_location] = temp   
                    dest_buf[next_output] = temp                     # and echo it to output 
                    next_qp += 1
                elif tag == self.MISS_TAG:
                    missed_word = src_buf[next_full_word]
                    next_full_word += 1
                    dict_location = hashLookupTable[(missed_word >> 10) & 0xFF] / 4
                    dictionary[dict_location] = missed_word
                    dest_buf[next_output] = missed_word
                else:
                    return -1 # fail, buffer is corrupted
                    #print "BAD TAG!!"

                next_tag += 1
                next_output += 1

            return next_output
        except:
            return -1

    
###########################################################
###########################################################
# testing area
###########################################################
###########################################################

#from struct import *

import sys
import time


def main():

    
    NUMBER_OF_ITERATIONS=1000
    w = WKdm()

#     src_buf_asm = [0]  * (w.PAGE_SIZE_IN_WORDS+100)
#     dest_buf_asm = [0] * (w.PAGE_SIZE_IN_WORDS+100)

    
    src_buf = [0]  * (w.PAGE_SIZE_IN_WORDS+100)
    dest_buf = [0] * (w.PAGE_SIZE_IN_WORDS+100)
    t=0


    for i in range(w.PAGE_SIZE_IN_WORDS):
        src_buf[i] = i * i + i
        if i % 10 == 0:
            src_buf[i] = 0
        elif i % 11 == 0:
            src_buf[i]=0xFFFFFFFF

    before = time.time()
    for i in range(NUMBER_OF_ITERATIONS):
        t += w.WKdm_compress(src_buf, dest_buf, w.PAGE_SIZE_IN_WORDS, 4096)
        t += w.WKdm_decompress(dest_buf, src_buf)
    total = time.time() - before
    
    print "Python timing: " + str(NUMBER_OF_ITERATIONS / total) + " compression / decompression pairs per second."


if __name__ == "__main__":
    main()
















# Volatility
# Copyright (C) Michael Ligh <michael.ligh@mnin.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details. 
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

import os
import volatility.plugins.common as common
import volatility.utils as utils 
import volatility.win32.tasks as tasks
import volatility.obj as obj
import volatility.debug as debug
import volatility.poolscan as poolscan
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

#--------------------------------------------------------------------------------
# Profile Modifications 
#--------------------------------------------------------------------------------

class PoolTrackTagOverlay(obj.ProfileModification):
    """Overlays for pool trackers"""

    conditions = {'os': lambda x: x == 'windows'}

    def modification(self, profile):
        profile.merge_overlay({
            '_POOL_TRACKER_TABLE': [ None, {
            'Key': [ None, ['String', dict(length = 4)]]
             }],
        })

#--------------------------------------------------------------------------------
# PoolTracker Plugin
#--------------------------------------------------------------------------------

class PoolTracker(common.AbstractWindowsCommand):
    """Show a summary of pool tag usage"""

    def __init__(self, config, *args, **kwargs):
        common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)
        config.add_option('TAGS', short_option = 't', help = 'Pool tag to find')
        config.add_option('TAGFILE', short_option = 'T', 
                help = 'Pool tag file (pooltag.txt)', default = None) 
        config.add_option('WHITELIST', short_option = 'W', 
                help = 'Apply whitelist (only show third party tags)', 
                default = False, action = "store_true")
        config.add_option('SHOW-FREE', short_option = 'F', 
                help = 'Show tags with no allocations', 
                default = False, action = "store_true")

    @staticmethod
    def is_valid_profile(profile):
        return (profile.metadata.get('os', 'unknown') == 'windows' and
                profile.metadata.get('major', 0) == 6)

    def calculate(self):
        kernel_space = utils.load_as(self._config)

        if not self.is_valid_profile(kernel_space.profile):
            debug.error("Windows XP/2003 does not track pool tags")

        knowntags = {}
        if self._config.TAGFILE and os.path.isfile(self._config.TAGFILE):
            taglines = open(self._config.TAGFILE).readlines()
            for tag in taglines:
                tag = tag.strip()
                if tag.startswith("rem") or tag.startswith(" ") or tag == "":
                    continue
                info = tag.split("-", 2)
                try:
                    key = info[0].strip()
                    driver = info[1].strip()
                    reason = info[2].strip()
                except IndexError:
                    continue
                knowntags[key] = (driver, reason)

        track_table = tasks.get_kdbg(kernel_space).PoolTrackTable

        # not really an address, this is just a trick to get 
        # a 32bit number on x86 and 64bit number on x64. the
        # size is always directly before the pool table. 
        table_size = obj.Object("address", offset = 
            track_table - kernel_space.profile.get_obj_size("address"), 
            vm = kernel_space
            )

        track_table = track_table.dereference_as("address")

        if not kernel_space.is_valid_address(track_table) or table_size > 100000:
            debug.error("Cannot find the table or its size is unexpected: {0}".format(table_size))

        entries = obj.Object("Array", targetType = "_POOL_TRACKER_TABLE", 
            offset = track_table, count = table_size, 
            vm = kernel_space
            )

        if self._config.TAGS:
            tags = [tag for tag in self._config.TAGS.split(",")]
        else:
            tags = []

        for entry in entries:

            if not self._config.SHOW_FREE:
                if entry.PagedBytes == 0 and entry.NonPagedBytes == 0:
                    continue

            if not tags or entry.Key in tags:
                try:
                    (driver, reason) = knowntags[str(entry.Key).strip()]
                    if self._config.WHITELIST:
                        continue
                except KeyError:
                    (driver, reason) = ("", "")
                yield entry, driver, reason

    def render_whitelist(self, outfd, data):

        for entry, driver, reason in data:
            if str(entry.Key) == "":
                continue
            outfd.write("{0} - {1} - {2}\n".format(entry.Key, driver, reason))

    def render_text(self, outfd, data):
        
        self.table_header(outfd, [("Tag", "6"), 
                                  ("NpAllocs", "8"), 
                                  ("NpFrees", "8"), 
                                  ("NpBytes", "8"), 
                                  ("PgAllocs", "8"), 
                                  ("PgFrees", "8"), 
                                  ("PgBytes", "8"), 
                                  ("Driver", "20"), 
                                  ("Reason", "")])

        for entry, driver, reason in data:
            if str(entry.Key) == "":
                continue

            self.table_row(outfd, entry.Key, entry.NonPagedAllocs, 
                entry.NonPagedFrees, entry.NonPagedBytes, entry.PagedAllocs, 
                entry.PagedFrees, entry.PagedBytes, 
                driver, reason)

    def unified_output(self, data):
        return TreeGrid([("Tag", str),
                       ("NpAllocs", int),
                       ("NpFrees", int),
                       ("NpBytes", int),
                       ("PgAllocs", int),
                       ("PgFrees", int),
                       ("PgBytes", int),
                       ("Driver", str),
                       ("Reason", str)],
                        self.generator(data))

    def generator(self, data):
        for entry, driver, reason in data:
            if str(entry.Key) == "":
                continue

            yield (0, [str(entry.Key), 
                int(entry.NonPagedAllocs),
                int(entry.NonPagedFrees), 
                int(entry.NonPagedBytes), 
                int(entry.PagedAllocs),
                int(entry.PagedFrees),
                int(entry.PagedBytes),
                str(driver), 
                str(reason)])

#--------------------------------------------------------------------------------
# Configurable PoolScanner Plugin
#--------------------------------------------------------------------------------

class GenericPoolScan(poolscan.SinglePoolScanner):
    """Configurable pool scanner"""

class PoolPeek(common.AbstractWindowsCommand):
    """Configurable pool scanner plugin"""

    def __init__(self, config, *args, **kwargs):
        common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)
        config.add_option('TAG', short_option = 't', 
                    help = 'Pool tag to find')   
        config.add_option('MIN-SIZE', short_option = 'm', 
                    type = 'int', 
                    help = 'Minimum size of the pool to find (default: 0)', 
                    default = 0)   
        config.add_option('MAX-SIZE', short_option = 'M', 
                    type = 'int', 
                    help = 'Maximum size of the pool to find (default: 4096)', 
                    default = 4096)   
        config.add_option('PAGED', short_option = 'P', 
                    help = 'Search in paged pools (default: False)', 
                    default = False, action = "store_true")

    def calculate(self):
        addr_space = utils.load_as(self._config)

        tag = self._config.TAG

        if tag == None:
            debug.error("You must enter a --tag to find")

        minsize = self._config.MIN_SIZE
        maxsize = self._config.MAX_SIZE 
        poolsize = lambda x : x >= minsize and x <= maxsize 

        if self._config.PAGED:
            paged = True
            non_paged = False
        else:
            paged = False
            non_paged = True

        scanner = GenericPoolScan()
        scanner.checks = [ 
                ('PoolTagCheck', dict(tag = tag)),
                ('CheckPoolSize', dict(condition = poolsize)),
                ('CheckPoolType', dict(paged = paged, non_paged = non_paged)),
                ]

        for offset in scanner.scan(addr_space):
            pool = obj.Object("_POOL_HEADER", offset = offset, vm = addr_space) 
            buf = addr_space.zread(offset, minsize)
            yield pool, buf

    def render_text(self, outfd, data):
        for pool, buf in data:
            pool_alignment = obj.VolMagic(pool.obj_vm).PoolAlignment.v()
            outfd.write("Pool Header: {0:#x}, Size: {1}\n".format(
                    pool.obj_offset, 
                    pool.BlockSize * pool_alignment))
            outfd.write("{0}\n".format("\n".join(
                    ["{0:#010x}  {1:<48}  {2}".format(pool.obj_offset + o, h, ''.join(c))
                    for o, h, c in utils.Hexdump(buf)
                    ])))
            outfd.write("\n")




# This file is part of Volatility.
# Copyright (C) 2007-2013 Volatility Foundation
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
"""

import volatility.plugins.linux.pslist as linux_pslist
from volatility.renderers.basic import Address
from volatility.renderers import TreeGrid
from collections import OrderedDict

class linux_pstree(linux_pslist.linux_pslist):
    '''Shows the parent/child relationship between processes'''

    def __init__(self, *args, **kwargs):
        self.procs = {}
        linux_pslist.linux_pslist.__init__(self, *args, **kwargs)

    def unified_output(self, data):
        return TreeGrid([("Offset",Address),
                        ("Name",str),
                        ("Level",str),
                         ("Pid",int),
                         ("Ppid",int),
                            ("Uid", int),
                            ("Gid",int),
                            ("Euid",int)],
                        self.generator(data))

    def generator(self, data):
        self.procs = OrderedDict()
        for task in data:
            self.recurse_task(task, 0, 0,self.procs)
        
        for offset,name,level,pid,ppid,uid,euid,gid in self.procs.values():
            if offset:
                yield(0,[Address(offset),
                         str(name),
                         str(level),
                         int(pid),
                         int(ppid),
                         int(uid),
                         int(gid),
                         int(euid)])

    def recurse_task(self,task,ppid,level,procs):
        """
        Fill a dictionnary with all the children of a given task(including itself)
        :param task: task that we want to get the children from
        :param ppid: pid of the parent task
        :param level: depth from the root task
        :param procs: dictionnary that we fill
        """
        if not procs.has_key(task.pid.v()):
            if task.mm:
                proc_name = task.comm
            else:
                proc_name = "[" + task.comm + "]"
            procs[task.pid.v()] = (task.obj_offset,proc_name,"." * level + proc_name,task.pid,ppid,task.uid,task.euid,task.gid)
            for child in task.children.list_of_type("task_struct", "sibling"):
                self.recurse_task(child,task.pid, level + 1,procs)

    def render_text(self, outfd, data):
        self.procs = OrderedDict()
        outfd.write("{0:20s} {1:15s} {2:15s}\n".format("Name", "Pid", "Uid"))
        for task in data:
            self.recurse_task(task, 0, 0, self.procs)
        
        for offset,_,proc_name,pid,_,uid,_,_ in self.procs.values():
            if offset:
                outfd.write("{0:20s} {1:15s} {2:15s}\n".format(proc_name, str(pid), str(uid or '')))    




# Volatility
# Copyright (C) 2009-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.utils as utils
import volatility.plugins.common as common
import volatility.cache as cache
import volatility.debug as debug
import volatility.obj as obj
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address
import datetime

class _DMP_HEADER(obj.CType):
    """A class for crash dumps"""

    @property
    def SystemUpTime(self):
        """Returns a string uptime"""

        # Some utilities write PAGEPAGE to this field when 
        # creating the dump header. 
        if self.m('SystemUpTime') == 0x4547415045474150:
            return obj.NoneObject("No uptime recorded")

        # 1 uptime is 100ns so convert that to microsec
        msec = self.m('SystemUpTime') / 10

        return datetime.timedelta(microseconds = msec)

class CrashInfoModification(obj.ProfileModification):
    """Applies overlays for crash dump headers"""

    conditions = {'os': lambda x: x == 'windows'}

    before = ["WindowsVTypes", "WindowsObjectClasses"]

    def modification(self, profile):
        profile.merge_overlay({
                '_DMP_HEADER' : [ None, {
                    'Comment' : [ None, ['String', dict(length = 128)]],
                    'DumpType' : [ None, ['Enumeration', dict(choices = {0x1: "Full Dump", 0x2: "Kernel Dump", 0x5: "BitMap Dump"})]],
                    'SystemTime' : [ None, ['WinTimeStamp', dict(is_utc = True)]],
                }],
                '_DMP_HEADER64' : [ None, {
                    'Comment' : [ None, ['String', dict(length = 128)]],
                    'DumpType' : [ None, ['Enumeration', dict(choices = {0x1: "Full Dump", 0x2: "Kernel Dump", 0x5: "BitMap Dump"})]],
                    'SystemTime' : [ None, ['WinTimeStamp', dict(is_utc = True)]],
                }],
            })

        ## Both x86 and x64 use the same structure for now, just
        ## so they can share the same SystemUpTime property.
        profile.object_classes.update({'_DMP_HEADER' : _DMP_HEADER, '_DMP_HEADER64' : _DMP_HEADER})

class CrashInfo(common.AbstractWindowsCommand):
    """Dump crash-dump information"""

    target_as = ['WindowsCrashDumpSpace32', 'WindowsCrashDumpSpace64', 'WindowsCrashDumpSpace64BitMap']

    @cache.CacheDecorator("tests/crashinfo")
    def calculate(self):
        """Determines the address space"""
        addr_space = utils.load_as(self._config, astype = 'physical')

        result = None
        adrs = addr_space
        while adrs:
            if adrs.__class__.__name__ in self.target_as:
                result = adrs
            adrs = adrs.base

        if result is None:
            debug.error("Memory Image could not be identified as {0}".format(self.target_as))

        return result

    def unified_output(self, data):
        return TreeGrid([("HeaderName", str),
                       ("Majorversion", Address),
                       ("Minorversion", Address),
                       ("KdSecondaryVersion", Address),
                       ("DirectoryTableBase", Address),
                       ("PfnDataBase", Address),
                       ("PsLoadedModuleList", Address),
                       ("PsActiveProcessHead", Address),
                       ("MachineImageType", Address),
                       ("NumberProcessors", Address),
                       ("BugCheckCode", Address),
                       ("PaeEnabled", Address),
                       ("KdDebuggerDataBlock", Address),
                       ("ProductType", Address),
                       ("SuiteMask", Address),
                       ("WriterStatus", Address),
                       ("Comment", str),
                       ("DumpType", str),
                       ("SystemTime", str),
                       ("SystemUpTime", str),
                       ("NumRuns", int)],
                        self.generator(data))

    def generator(self, data):
        hdr = data.get_header()
        pae = -1
        if hdr.obj_name != "_DMP_HEADER64":
            pae = hdr.PaeEnabled
        yield (0, [str(hdr.obj_name),
                    Address(hdr.MajorVersion),
                    Address(hdr.MinorVersion),
                    Address(hdr.KdSecondaryVersion),
                    Address(hdr.DirectoryTableBase),
                    Address(hdr.PfnDataBase),
                    Address(hdr.PsLoadedModuleList),
                    Address(hdr.PsActiveProcessHead),
                    Address(hdr.MachineImageType),
                    Address(hdr.NumberProcessors),
                    Address(hdr.BugCheckCode),
                    Address(pae),
                    Address(hdr.KdDebuggerDataBlock),
                    Address(hdr.ProductType),
                    Address(hdr.SuiteMask),
                    Address(hdr.WriterStatus),
                    str(hdr.Comment),
                    str(hdr.DumpType),
                    str(hdr.SystemTime or ''),
                    str(hdr.SystemUpTime or ''),
                    len(data.get_runs())])


    def render_text(self, outfd, data):
        """Renders the crashdump header as text"""

        hdr = data.get_header()
        runs = data.get_runs()

        outfd.write("{0}:\n".format(hdr.obj_name))
        outfd.write(" Majorversion:         0x{0:08x} ({1})\n".format(hdr.MajorVersion, hdr.MajorVersion))
        outfd.write(" Minorversion:         0x{0:08x} ({1})\n".format(hdr.MinorVersion, hdr.MinorVersion))
        outfd.write(" KdSecondaryVersion    0x{0:08x}\n".format(hdr.KdSecondaryVersion))
        outfd.write(" DirectoryTableBase    0x{0:08x}\n".format(hdr.DirectoryTableBase))
        outfd.write(" PfnDataBase           0x{0:08x}\n".format(hdr.PfnDataBase))
        outfd.write(" PsLoadedModuleList    0x{0:08x}\n".format(hdr.PsLoadedModuleList))
        outfd.write(" PsActiveProcessHead   0x{0:08x}\n".format(hdr.PsActiveProcessHead))
        outfd.write(" MachineImageType      0x{0:08x}\n".format(hdr.MachineImageType))
        outfd.write(" NumberProcessors      0x{0:08x}\n".format(hdr.NumberProcessors))
        outfd.write(" BugCheckCode          0x{0:08x}\n".format(hdr.BugCheckCode))
        if hdr.obj_name != "_DMP_HEADER64":
            outfd.write(" PaeEnabled            0x{0:08x}\n".format(hdr.PaeEnabled))
        outfd.write(" KdDebuggerDataBlock   0x{0:08x}\n".format(hdr.KdDebuggerDataBlock))
        outfd.write(" ProductType           0x{0:08x}\n".format(hdr.ProductType))
        outfd.write(" SuiteMask             0x{0:08x}\n".format(hdr.SuiteMask))
        outfd.write(" WriterStatus          0x{0:08x}\n".format(hdr.WriterStatus))
        outfd.write(" Comment               {0}\n".format(hdr.Comment))
        outfd.write(" DumpType              {0}\n".format(hdr.DumpType))
        outfd.write(" SystemTime            {0}\n".format(str(hdr.SystemTime or '')))
        outfd.write(" SystemUpTime          {0}\n".format(str(hdr.SystemUpTime or '')))
        outfd.write("\nPhysical Memory Description:\n")
        outfd.write("Number of runs: {0}\n".format(len(runs)))
        outfd.write("FileOffset    Start Address    Length\n")
        foffset = 0x1000
        if hdr.obj_name == "_DMP_HEADER64":
            foffset = 0x2000
        run = []

        ## FIXME. These runs differ for x86 vs x64. This is a reminder
        ## for MHL or AW to fix it. 

        for run in runs:
            outfd.write("{0:08x}      {1:08x}         {2:08x}\n".format(foffset, run[0], run[2]))
            foffset += (run[2])
        outfd.write("{0:08x}      {1:08x}\n".format(foffset - 0x1000, (run[0] + run[2] - 0x1000)))



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""
import volatility.obj as obj
import volatility.plugins.mac.common as common
import volatility.plugins.mac.mount as mac_mount
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class mac_list_files(common.AbstractMacCommand):
    """ Lists files in the file cache """

    def calculate(self):
        common.set_plugin_members(self)

        mounts = mac_mount.mac_mount(self._config).calculate()

        seen  = {}
        paths = {}

        for mount in mounts:
            vnode = mount.mnt_vnodelist.tqh_first

            while vnode:
                if vnode.v() in seen:
                    break
 
                seen[vnode.v()] = 1

                if vnode.v_flag.v() & 0x000001 != 0:
                    yield vnode, vnode.full_path()
                    
                    fname = ""
                    parent_vnode = None
                else:
                    fname = str(vnode.v_name.dereference() or '')
                    parent_vnode = vnode.v_parent

                if parent_vnode != None and fname != "":
                    parent_key = parent_vnode.v()

                    # if not then calc full path and store in cache
                    if not parent_key in paths:    
                        paths[parent_key] = parent_vnode.full_path()

                    if paths[parent_key] == "/":
                        sep = ""
                    else:
                        sep = "/"

                    # figure out our full path and store it
                    path = paths[parent_key] + sep + fname
                    paths[vnode.v()] = path
                    
                    yield vnode, path

                vnode = vnode.v_mntvnodes.tqe_next        
 
    def unified_output(self, data):
        return TreeGrid([("Offset (V)", Address),
                         ("File Path", str),
                         ], self.generator(data))

    def generator(self, data):
        for vnode, path in data:
            yield (0, [
                Address(vnode.v()),
                str(path),
            ])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Offset (V)", "[addrpad]"), ("File Path", "")])
        for vnode, path in data:
            self.table_row(outfd, vnode.v(), path)    

